// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/api/intime/os_main.h $
// $Rev: 43206 $
// $Date: 2018-07-08 17:03:14 -0500 (Sun, 08 Jul 2018) $

// INtime: API Library: header file

#ifndef __OS_MAIN_H__
#define __OS_MAIN_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "os_common.h"
#include "rt.h"



// #defines *******************************************************************

#define	OS_FD_ENCODE(i)						( OS_FD_ENCODE_INDEX((i))	\
											| OS_FD_ENCODE_FLAG(1)		\
											| OS_FD_ENCODE_COUNTER(os_api_global.fd_counter))

#define	OS_FD_DECODE_INDEX(v)				GSC_FIELD_DECODE((v),  7, 0)
#define	OS_FD_ENCODE_INDEX(v)				GSC_FIELD_ENCODE((v),  7, 0)
#define	OS_FD_ENCODE_FLAG(v)				GSC_FIELD_ENCODE((v),  8, 8)
#define	OS_FD_ENCODE_COUNTER(v)				GSC_FIELD_ENCODE((v), 16, 9)

#define	sizeofarray(a)						(sizeof((a)) / sizeof((a)[0]))



// data types *****************************************************************

typedef struct
{
	KNHANDLE	sem;
	u8			area[KN_SEMAPHORE_SIZE];
} os_sem_t;

typedef struct
{
	// Must have global lock to modify this structure!
	os_sem_t	sem;		// control access to this structure
	RTHANDLE	dev_mbox;	// Request messages are sent here.
	int			fd;			// an encoded value
} os_file_t;

typedef struct
{
	RTHANDLE	mbox;
} os_mbox_t;

typedef struct
{
	int			initialized;// 0 = not done, !0 = done
	os_sem_t	sem;		// our global lock
	u16			fd_counter;	// encoded into file descriptors used externally
	os_file_t	files[256];
} os_api_global_t;




// variables ******************************************************************

extern	os_api_global_t	os_api_global;



// prototypes *****************************************************************

int		os_api_close(int fd);
int		os_api_init(void);
int		os_api_ioctl(int fd, int cmd, void* arg);
int		os_api_open(int index, int share, const char* base);
int		os_api_read(int fd, void* dst, size_t bytes);
int		os_api_write(int fd, const void* src, size_t bytes);

// These are for internal use only.
int		os_file_create(os_file_t** fp);
int		os_file_destroy(os_file_t** fp);
int		os_file_lock(int fd, os_file_t** fp);
int		os_file_unlock(os_file_t** fp);

int		os_get_error_status(void);

int		os_mem_handle_create(void* ptr, int bytes, u32* offset, RTHANDLE* handle);
int		os_mem_handle_destroy(RTHANDLE* mem);

int		os_mbox_alloc(os_mbox_t** mbox);
int		os_mbox_free(os_mbox_t** mbox);

int		os_msg_alloc(os_msg_t** msg);
int		os_msg_free(os_msg_t** msg);

int		os_sem_create(os_sem_t* sem);
int		os_sem_destroy(os_sem_t* sem);
int		os_sem_lock(os_sem_t* sem);
int		os_sem_unlock(os_sem_t* sem);

void	os_validate(void);						// must define __OS_VALIDATE__ to use



#endif
