// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/api/intime/os_mem.c $
// $Rev: 43206 $
// $Date: 2018-07-08 17:03:14 -0500 (Sun, 08 Jul 2018) $

// INtime: API Library: source file

#include "main.h"



//*****************************************************************************
// Return Values:
//	0  = success
//	<0 = failure, value is -errno or -INtime error code
//	>0 = never returned
int os_mem_handle_create(void* ptr, int bytes, u32* offset, RTHANDLE* mem)
{
	RTHANDLE	handle;
	u32			off;
	int			ret;

	for (;;)	// A convenience loop.
	{
		if ((ptr == NULL) || (bytes <= 0) || (offset == NULL) || (mem == NULL))
		{
			ret	= -EINVAL;
			break;
		}

		off		= (u32) ptr % 4096;
		bytes	+= off;
		ptr		= (void*) ((u8*) ptr - off);
		handle	= CreateRtMemoryHandle(ptr, bytes);

		if (handle == BAD_RTHANDLE)
		{
			offset[0]	= 0;
			mem[0]		= handle;
			ret			= os_get_error_status();
			break;
		}

		offset[0]	= off;
		mem[0]		= handle;
		ret			= 0;
		break;
	}

	return(ret);
}



//*****************************************************************************
// Return Values:
//	0  = success
//	<0 = failure, value is -errno or -INtime error code
//	>0 = never returned
int os_mem_handle_destroy(RTHANDLE* mem)
{
	BOOLEAN	tst;
	int		ret;

	for (;;)	// A convenience loop.
	{
		if (mem == NULL)
		{
			// There is nothing to do.
			ret	= 0;
			break;
		}

		if (mem[0] == BAD_RTHANDLE)
		{
			// There is nothing to do.
			ret	= 0;
			break;
		}

		tst	= DeleteRtMemoryHandle(mem[0]);

		if (tst == 0)
		{
			ret	= os_get_error_status();
			break;
		}

		mem[0]	= BAD_RTHANDLE;
		ret		= 0;
		break;
	}

	return(ret);
}


