// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/api/intime/os_sem.c $
// $Rev: 43206 $
// $Date: 2018-07-08 17:03:14 -0500 (Sun, 08 Jul 2018) $

// INtime: API Library: source file

#include "main.h"




//*****************************************************************************
// Return Values:
//	0  = success
//	<0 = failure, value is -errno or -INtime error code
//	>0 = never returned
int os_sem_create(os_sem_t* sem)
{
	int	ret	= 0;

	for (;;)	// convenience loop
	{
		if (sem == NULL)
		{
			ret	= -EINVAL;
			break;
		}

		memset(sem, 0, sizeof(os_sem_t));
		sem->sem	= knCreateRtSemaphore(sem->area, KN_REGION | KN_ONE_UNIT);

		if (sem->sem == 0xffffffff)
		{
			memset(sem, 0, sizeof(os_sem_t));
			ret	= os_get_error_status();
			break;
		}

		break;
	}

	return(ret);
}



//*****************************************************************************
// Return Values:
//	0  = success
//	<0 = failure, value is -errno or -INtime error code
//	>0 = never returned
int os_sem_destroy(os_sem_t* sem)
{
	int	ret		= 0;
	
	for (;;)	// convenience loop
	{
		if (sem == NULL)
		{
			ret	= -EINVAL;
			break;
		}

		if (sem->sem == 0)
		{
			ret	= -EINVAL;
			memset(sem, 0, sizeof(os_sem_t));
			break;
		}

		knDeleteRtSemaphore(sem->sem);
		memset(sem, 0, sizeof(os_sem_t));
		break;
	}

	return(ret);
}



//*****************************************************************************
// Return Values:
//	0  = success
//	<0 = failure, value is -errno or -INtime error code
//	>0 = never returned
int os_sem_lock(os_sem_t* sem)
{
	int		ret		= 0;
	BOOLEAN	sts;

	for (;;)	// convenience loop
	{
		if (sem == NULL)
		{
			ret	= -EINVAL;
			break;
		}

		if (sem->sem == 0)
		{
			ret	= -EINVAL;
			break;
		}

		sts	= knWaitForRtSemaphore(sem->sem, KN_WAIT_FOREVER);

		if (sts == FALSE)
		{
			ret	= os_get_error_status();
			break;
		}

		break;
	}

	return(ret);
}



//*****************************************************************************
// Return Values:
//	0  = success
//	<0 = failure, value is -errno or -INtime error code
//	>0 = never returned
int os_sem_unlock(os_sem_t* sem)
{
	int	ret	= 0;

	for (;;)	// convenience loop
	{
		if (sem == NULL)
		{
			ret	= -EINVAL;
			break;
		}

		if (sem->sem == 0)
		{
			ret	= -EINVAL;
			break;
		}

		knReleaseRtSemaphore(sem->sem);
		break;
	}

	return(ret);
}


