// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/driver/device.c $
// $Rev: 43266 $
// $Date: 2018-07-09 17:21:40 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Device Driver: source file

#include "main.h"



// variables ******************************************************************

const gsc_dev_id_t	dev_id_list[]	=
{
	// model		Vendor	Device	SubVen	SubDev	type
	{ "16AO4MF",	0x10B5, 0x9080, 0x10B5, 0x3064,	GSC_DEV_TYPE_16AO4MF	},
	{ NULL }
};



// ****************************************************************************
static void _reg_diopcr_compute(chan_data_t* chan)
{
	if (chan->index == 0)
		chan->cache.reg_diopcr	= 1;
	else
		chan->cache.reg_diopcr	= 0;
}



// ****************************************************************************
static void _sync_clk_out_compute(chan_data_t* chan)
{
	if (chan->index == 0)
		chan->cache.sync_clk_out	= 1;
	else
		chan->cache.sync_clk_out	= 0;
}



// ****************************************************************************
static void _channel_data_create(dev_data_t* dev, int index)
{
	#define FW_VADDR(d,i,r)		GSC_VADDR((d),GSC_REG_OFFSET((r))+((i)*(0x40)))

	chan_data_t*	chan	= &dev->channel[index];

	os_sem_create(&chan->sem);
	chan->dev			= dev;
	chan->index			= index;
	chan->dev_index		= dev->board_index * 4 + index;

	// Initialize register access.
	chan->vaddr.gsc_accr_32		= FW_VADDR(dev, index, AO4MF_GSC_ACCR);	// Adjustable Clock Control Register
	chan->vaddr.gsc_bcr_32		= FW_VADDR(dev, index, AO4MF_GSC_BCR);	// Board Control Register
	chan->vaddr.gsc_bor_32		= FW_VADDR(dev, index, AO4MF_GSC_BOR);	// Buffer Operations Register
	chan->vaddr.gsc_diopcr_32	= FW_VADDR(dev, index, AO4MF_GSC_DIOPCR);// Digital I/O Port Control Register
	chan->vaddr.gsc_icscr_32	= FW_VADDR(dev, index, AO4MF_GSC_ICSCR);// Internal Clock Source Control Register
	chan->vaddr.gsc_odbr_32		= FW_VADDR(dev, index, AO4MF_GSC_ODBR);	// Output Data Buffer Register
	chan->vaddr.gsc_srcr_32		= FW_VADDR(dev, index, AO4MF_GSC_SRCR);	// Sample Rate Control Register

	// Check feature options.
	_reg_diopcr_compute(chan);
	_sync_clk_out_compute(chan);
}



/******************************************************************************
*
*	Function:	dev_device_create
*
*	Purpose:
*
*		Perform a one-time initialization of the device's main data structure.
*
*	Arguments:
*
*		dev		The structure for the device.
*
*	Returned:
*
*		0		All is well.
*		< 0		An appropriate error status.
*
******************************************************************************/

int dev_device_create(dev_data_t* dev)
{
	static const gsc_bar_maps_t	bar_map	=
	{
		{
			// mem	io	rw
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 0: PLX registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 1: PLX registers, I/O mapped
			{ 1,	0,	GSC_REG_TYPE_ACCESS_RW },	// BAR 2: GSC registers, memory mapped
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 3: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO },	// BAR 4: unused
			{ 0,	0,	GSC_REG_TYPE_ACCESS_RO }	// BAR 5: unused
		}
	};

	u32	dma;
	int	ret;

	for (;;)	// A convenience loop.
	{
		// Verify some macro contents.
		ret	= gsc_macro_test_base_name(AO4MF_BASE_NAME);
		if (ret)	break;

		ret	= gsc_macro_test_model();
		if (ret)	break;

		// PCI setup.
		ret	= os_pci_dev_enable(&dev->pci);
		if (ret)	break;

		ret	= os_pci_master_set(&dev->pci);
		if (ret)	break;

		// Control ISR access to the device and data structure.
		ret	= os_spinlock_create(&dev->spinlock);
		if (ret)	break;

		// Control access to the device and data structure.
		ret	= os_sem_create(&dev->sem);
		if (ret)	break;

		// Access the BAR regions.
		ret	= gsc_bar_create(dev, &dev->bar, &bar_map);
		if (ret)	break;

		// Data cache initialization.
		dev->cache.auto_cal_ms	= 3000;
		dev->cache.channel_qty	= 4;
		dev->cache.fref_adj		= 16000000;
		dev->cache.fref_master	= 27000000;
		dev->cache.fsamp_max	= 400000;
		dev->cache.fsamp_min	= 244;
		dev->cache.init_ms		= 200;
		dev->cache.nclk_max		= 511;
		dev->cache.nclk_min		= 0;
		dev->cache.nrate_max	= 0xFFFF;
		dev->cache.nrate_min	= 68;

		_channel_data_create(dev, 0);
		_channel_data_create(dev, 1);
		_channel_data_create(dev, 2);
		_channel_data_create(dev, 3);

		// Initialize additional resources.
		ret	= dev_irq_create(dev);
		if (ret)	break;

		ret	= dev_io_create(dev, 0);
		if (ret)	break;

		ret	= dev_io_create(dev, 1);
		if (ret)	break;

		ret	= dev_io_create(dev, 2);
		if (ret)	break;

		ret	= dev_io_create(dev, 3);
		if (ret)	break;

		dma	= GSC_DMA_SEL_DYNAMIC
			| GSC_DMA_CAP_DMA_WRITE;
		ret	= gsc_dma_create(dev, dma, dma);
		break;
	}

	return(ret);
}



//*****************************************************************************
static void _chan_data_destroy(dev_data_t* dev, int index)
{
	chan_data_t*	chan	= &dev->channel[index];

	os_sem_destroy(&chan->sem);
	memset(chan, 0, sizeof(chan_data_t));
}



/******************************************************************************
*
*	Function:	dev_device_destroy
*
*	Purpose:
*
*		Do everything needed to release the referenced device.
*
*	Arguments:
*
*		dev		The partial data for the device of interest.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_device_destroy(dev_data_t* dev)
{
	if (dev)
	{
		gsc_dma_destroy(dev);

		dev_io_destroy(dev, 0);
		dev_io_destroy(dev, 1);
		dev_io_destroy(dev, 2);
		dev_io_destroy(dev, 3);

		dev_irq_destroy(dev);

		_chan_data_destroy(dev, 0);
		_chan_data_destroy(dev, 1);
		_chan_data_destroy(dev, 2);
		_chan_data_destroy(dev, 3);

		gsc_bar_destroy(&dev->bar);

		os_sem_destroy(&dev->sem);
		os_spinlock_destroy(&dev->spinlock);

		os_pci_master_clear(&dev->pci);
		os_pci_dev_disable(&dev->pci);
	}
}


