// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/driver/io.c $
// $Rev: 43266 $
// $Date: 2018-07-09 17:21:40 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Device Driver: source file

#include "main.h"



/******************************************************************************
*
*	Function:	dev_io_close
*
*	Purpose:
*
*		Cleanup the I/O stuff for the channel as it is being closed.
*
*	Arguments:
*
*		chan	The channel data structure.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_io_close(chan_data_t* chan)
{
	chan->tx.dma_channel	= NULL;
}



/******************************************************************************
*
*	Function:	dev_io_create
*
*	Purpose:
*
*		Perform I/O based initialization as the driver is being loaded.
*
*	Arguments:
*
*		dev		The data for the device of interest.
*
*		index	The index of the channel to access.
*
*	Returned:
*
*		0		All went well.
*		< 0		An appropriate error code.
*
******************************************************************************/

int dev_io_create(dev_data_t* dev, int index)
{
	chan_data_t*		chan	= &dev->channel[index];
	int					ret;

	ret							= gsc_io_create(chan, &chan->tx, _32K);
	chan->tx.io_reg_offset		= 0x18 + (index * 0x40);
	chan->tx.io_reg_vaddr		= chan->vaddr.gsc_odbr_32;
	return(ret);
}



/******************************************************************************
*
*	Function:	dev_io_destroy
*
*	Purpose:
*
*		Perform I/O based cleanup as the driver is being unloaded.
*
*	Arguments:
*
*		dev		The data for the device of interest.
*
*		index	The index of the channel to access.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dev_io_destroy(dev_data_t* dev, int index)
{
	chan_data_t*	chan	= &dev->channel[index];

	dev_io_close(chan);	// Just in case.
	gsc_io_destroy(chan, &chan->tx);
}



/******************************************************************************
*
*	Function:	dev_io_open
*
*	Purpose:
*
*		Perform initialization work for a device that is being opened.
*
*	Arguments.
*
*		chan	The channel data structure.
*
*	Returned:
*
*		0	All went well.
*		< 0	An appropriate error code.
*
******************************************************************************/

int dev_io_open(chan_data_t* chan)
{
	dev_io_close(chan);	// Just in case.
	return(0);
}


