// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/driver/ioctl.c $
// $Rev: 43267 $
// $Date: 2018-07-09 17:24:11 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Device Driver: source file

#include "main.h"



//*****************************************************************************
static int _reg_read(chan_data_t* chan, gsc_reg_t* arg)
{
	u16			offset;
	int			ret;
	u16			size;
	gsc_reg_t	tmp;
	u16			type	= GSC_REG_TYPE(arg->reg);

	if (type == GSC_REG_TYPE_BAR2)
	{
		size		= GSC_REG_SIZE(arg->reg);
		offset		= GSC_REG_OFFSET(arg->reg);
		offset		+= chan->index * 0x40;		// Adjust for channel index.
		tmp.reg		= GSC_REG_ENCODE(type, size, offset);
		tmp.value	= arg->value;
		tmp.mask	= arg->mask;
		ret			= gsc_reg_read_ioctl(chan, &tmp);
		arg->value	= tmp.value;
	}
	else
	{
		ret	= gsc_reg_read_ioctl(chan, arg);
	}

	return(ret);
}



//*****************************************************************************
static int _reg_write(chan_data_t* chan, gsc_reg_t* arg)
{
	u16			offset;
	int			ret;
	u16			size;
	gsc_reg_t	tmp;
	u16			type	= GSC_REG_TYPE(arg->reg);

	if (type == GSC_REG_TYPE_BAR2)
	{
		size		= GSC_REG_SIZE(arg->reg);
		offset		= GSC_REG_OFFSET(arg->reg);
		offset		+= chan->index * 0x40;		// Adjust for channel index.
		tmp.reg		= GSC_REG_ENCODE(type, size, offset);
		tmp.value	= arg->value;
		tmp.mask	= arg->mask;
		ret			= gsc_reg_write_ioctl(chan, &tmp);
	}
	else
	{
		ret	= gsc_reg_write_ioctl(chan, arg);
	}

	return(ret);
}



//*****************************************************************************
static int _reg_mod(chan_data_t* chan, gsc_reg_t* arg)
{
	u16			offset;
	int			ret;
	u16			size;
	gsc_reg_t	tmp;
	u16			type	= GSC_REG_TYPE(arg->reg);

	if (type == GSC_REG_TYPE_BAR2)
	{
		size		= GSC_REG_SIZE(arg->reg);
		offset		= GSC_REG_OFFSET(arg->reg);
		offset		+= chan->index * 0x40;		// Adjust for channel index.
		tmp.reg		= GSC_REG_ENCODE(type, size, offset);
		tmp.value	= arg->value;
		tmp.mask	= arg->mask;
		ret			= gsc_reg_mod_ioctl(chan, &tmp);
	}
	else
	{
		ret	= gsc_reg_mod_ioctl(chan, arg);
	}

	return(ret);
}



//*****************************************************************************
static int _query(chan_data_t* chan, s32* arg)
{
	dev_data_t*	dev	= chan->dev;
	int			ret	= 0;

	switch (arg[0])
	{
		default:						ret		= -EINVAL;					break;
		case AO4MF_QUERY_AUT_CAL_MS:	arg[0]	= dev->cache.auto_cal_ms;	break;
		case AO4MF_QUERY_CHANNEL_QTY:	arg[0]	= dev->cache.channel_qty;	break;
		case AO4MF_QUERY_COUNT:			arg[0]	= AO4MF_IOCTL_QUERY_LAST;	break;
		case AO4MF_QUERY_DEVICE_TYPE:	arg[0]	= dev->board_type;			break;
		case AO4MF_QUERY_FREF_ADJ:		arg[0]	= dev->cache.fref_adj;		break;
		case AO4MF_QUERY_FREF_MASTER:	arg[0]	= dev->cache.fref_master;	break;
		case AO4MF_QUERY_FSAMP_MAX:		arg[0]	= dev->cache.fsamp_max;		break;
		case AO4MF_QUERY_FSAMP_MIN:		arg[0]	= dev->cache.fsamp_min;		break;
		case AO4MF_QUERY_INDEX_CHANNEL:	arg[0]	= chan->index;				break;
		case AO4MF_QUERY_INDEX_DEVICE:	arg[0]	= chan->dev_index;			break;
		case AO4MF_QUERY_INIT_MS:		arg[0]	= dev->cache.init_ms;		break;
		case AO4MF_QUERY_NCLK_MAX:		arg[0]	= dev->cache.nclk_max;		break;
		case AO4MF_QUERY_NCLK_MIN:		arg[0]	= dev->cache.nclk_min;		break;
		case AO4MF_QUERY_NRATE_MAX:		arg[0]	= dev->cache.nrate_max;		break;
		case AO4MF_QUERY_NRATE_MIN:		arg[0]	= dev->cache.nrate_min;		break;
		case AO4MF_QUERY_REG_DIOPCR:	arg[0]	= chan->cache.reg_diopcr;	break;
		case AO4MF_QUERY_SYNC_CLK_OUT:	arg[0]	= chan->cache.sync_clk_out;	break;
	}

	return(ret);
}



//*****************************************************************************
int initialize_ioctl(chan_data_t* chan, s32* arg)
{
	dev_data_t*	dev	= chan->dev;
	long		ms;
	int			ret;

	if (chan->index == 0)
	{
		ret	= 0;

		// Initialization on channel 0 clears the IRQ status bit on the other three
		// channels. For this reason alone the initialize bit is unused. Instead,
		// initialization is performed manually.

		os_reg_mem_tx_u32(dev, chan->vaddr.gsc_bcr_32,		0x00000810);
		os_reg_mem_tx_u32(dev, chan->vaddr.gsc_diopcr_32,	0x00000000);
		os_reg_mem_tx_u32(dev, chan->vaddr.gsc_srcr_32,		0x0000005A);
		os_reg_mem_tx_u32(dev, chan->vaddr.gsc_bor_32,		0x0000340D);
		os_reg_mem_tx_u32(dev, chan->vaddr.gsc_accr_32,		0x00000000);
		os_reg_mem_tx_u32(dev, chan->vaddr.gsc_icscr_32,		0x00000000);
		os_reg_mem_tx_u32(dev, chan->vaddr.gsc_bor_32,		0x00003C0D); // 340D + clear buffer
		os_time_sleep_ms(10);

		// Simulate generation of an Initialize Done IRQ having been generated.
		gsc_wait_resume_irq_gsc(chan, AO4MF_WAIT_GSC_INIT_DONE);
	}
	else
	{
		ms	= chan->dev->cache.init_ms + 5000;
		irq_reg_mod(chan, chan->vaddr.gsc_bcr_32, D15, D15);
		ret	= gsc_poll_u32(chan->dev, ms, chan->vaddr.gsc_bcr_32, D15, 0);

		if (ret)
		{
			printf(	"%s: INITIALIZATION TIMED OUT(%ld ms)\n",
					dev->model,
					(long) ms);
		}
	}

	// Initialize the software settings.
	chan->tx.bytes_per_sample	= 4;
	chan->tx.io_mode			= AO4MF_IO_MODE_DEFAULT;
	chan->tx.pio_threshold		= 32;
	chan->tx.timeout_s			= AO4MF_IO_TIMEOUT_DEFAULT;
	return(ret);
}



//*****************************************************************************
static int _irq_sel(chan_data_t* chan, s32* arg)
{
	static const s32	options[]	=
	{
		AO4MF_IRQ_SEL_NONE_INIT_DONE,
		AO4MF_IRQ_SEL_AUTO_CAL_DONE,
		AO4MF_IRQ_SEL_AO_BUF_EMPTY,
		AO4MF_IRQ_SEL_AO_BUF_1Q_FULL,
		AO4MF_IRQ_SEL_AO_BUF_3Q_FULL,
		AO4MF_IRQ_SEL_BURST_TRIG_READY,
		AO4MF_IRQ_SEL_LOAD_READY,
		AO4MF_IRQ_SEL_LOAD_READY_END,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(chan->dev, arg, options, chan->vaddr.gsc_bcr_32, 10, 8);
	return(ret);
}



//*****************************************************************************
static int _auto_cal_start(chan_data_t* chan, void* arg)
{
	#define AUTO_CAL_START	D13
	#define AUTO_CAL_MASK	(D12 | D13)

	// Initiate auto-calibration.
	os_reg_mem_mx_u32(NULL, chan->vaddr.gsc_bcr_32, AUTO_CAL_START, AUTO_CAL_MASK);
	return(0);
}



//*****************************************************************************
static int _auto_cal(chan_data_t* chan, s32* arg)
{
	#define AUTO_CAL_FAIL			D14

	dev_data_t*	dev	= chan->dev;
	long		ms;
	u32			reg;
	int			ret;
	os_sem_t	sem;
	int			sts;
	s32			val;
	gsc_wait_t	wt;

	// Safely enable the Auto-Cal Done interrupt.
	val	= AO4MF_IRQ_SEL_AUTO_CAL_DONE;
	_irq_sel(chan, &val);

	// Wait for the local interrupt.
	os_sem_create(&sem);	// dummy, required for wait operations.
	ms				= dev->cache.auto_cal_ms + 5000;
	memset(&wt, 0, sizeof(wt));
	wt.flags		= GSC_WAIT_FLAG_INTERNAL;
	wt.gsc			= AO4MF_WAIT_GSC_AUTO_CAL_DONE;
	wt.timeout_ms	= ms;
	ret				= gsc_wait_event(chan, &wt, _auto_cal_start, NULL, &sem);
	os_sem_destroy(&sem);

	if (wt.flags & GSC_WAIT_FLAG_TIMEOUT)
	{
		ret	= ret ? ret : -ETIMEDOUT;
		printf(	"%s: AUTO-CALIBRATION TIMED OUT (%ld ms).\n",
				dev->model,
				(long) ms);
	}

	// Manually wait for completion.
	sts	= gsc_poll_u32(chan->dev, ms, chan->vaddr.gsc_bcr_32, AUTO_CAL_MASK, 0);
	ret	= ret ? ret : sts;

	// Check the completion status.
	reg	= os_reg_mem_rx_u32(dev, chan->vaddr.gsc_bcr_32);

	if (ret)
	{
	}
	else if (reg & AUTO_CAL_START)
	{
		ret	= ret ? ret : -EIO;
		printf(	"%s: AUTO-CALIBRATION DID NOT COMPLETE (%ld ms)\n",
				dev->model,
				(long) ms);
	}
	else if (reg & AUTO_CAL_FAIL)
	{
		ret	= ret ? ret : -EIO;
		printf(	"%s: AUTO-CALIBRATION FAILED (%ld ms)\n",
				dev->model,
				(long) ms);
	}
	else
	{
		// Wait for settling.
		os_time_sleep_ms(100);
	}

	return(ret);
}



//*****************************************************************************
static int _auto_cal_sts(chan_data_t* chan, s32* arg)
{
	u32	reg;

	reg	= os_reg_mem_rx_u32(chan->dev, chan->vaddr.gsc_bcr_32);

	if (reg & AUTO_CAL_START)
		arg[0]	= AO4MF_AUTO_CAL_STS_ACTIVE;
	else if (reg & AUTO_CAL_FAIL)
		arg[0]	= AO4MF_AUTO_CAL_STS_FAIL;
	else
		arg[0]	= AO4MF_AUTO_CAL_STS_PASS;

	return(0);
}



//*****************************************************************************
static int _ao_buf_clear(chan_data_t* chan, s32* arg)
{
	os_reg_mem_mx_u32(chan->dev, chan->vaddr.gsc_bor_32, D11, D11);
	return(0);
}



//*****************************************************************************
static int _ao_buf_load_ready(chan_data_t* chan, s32* arg)
{
	static const s32	options[]	=
	{
		AO4MF_AO_BUF_LOAD_READY_NO,
		AO4MF_AO_BUF_LOAD_READY_YES,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(chan->dev, arg, options, chan->vaddr.gsc_bor_32, 10, 10);
	return(ret);
}



//*****************************************************************************
static int _ao_buf_load_req(chan_data_t* chan, s32* arg)
{
	os_reg_mem_mx_u32(chan->dev, chan->vaddr.gsc_bor_32, D9, D9);
	return(0);
}



//*****************************************************************************
static int _ao_buf_mode(chan_data_t* chan, s32* arg)
{
	static const s32	options[]	=
	{
		AO4MF_AO_BUF_MODE_OPEN,
		AO4MF_AO_BUF_MODE_CIRC,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(chan->dev, arg, options, chan->vaddr.gsc_bor_32, 8, 8);
	return(ret);
}



//*****************************************************************************
static int _ao_buf_size(chan_data_t* chan, s32* arg)
{
	static const s32	options[]	=
	{
		AO4MF_AO_BUF_SIZE_4,
		AO4MF_AO_BUF_SIZE_8,
		AO4MF_AO_BUF_SIZE_16,
		AO4MF_AO_BUF_SIZE_32,
		AO4MF_AO_BUF_SIZE_64,
		AO4MF_AO_BUF_SIZE_128,
		AO4MF_AO_BUF_SIZE_256,
		AO4MF_AO_BUF_SIZE_512,
		AO4MF_AO_BUF_SIZE_1K,
		AO4MF_AO_BUF_SIZE_2K,
		AO4MF_AO_BUF_SIZE_4K,
		AO4MF_AO_BUF_SIZE_8K,
		AO4MF_AO_BUF_SIZE_16K,
		AO4MF_AO_BUF_SIZE_32K,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(chan->dev, arg, options, chan->vaddr.gsc_bor_32, 3, 0);
	return(ret);
}



//*****************************************************************************
static int _ao_buf_status(chan_data_t* chan, s32* arg)
{
	u32	reg;

	reg	= os_reg_mem_rx_u32(chan->dev, chan->vaddr.gsc_bor_32);

	if (reg & D12)
		arg[0]	= AO4MF_AO_BUF_STATUS_EMPTY;
	else if (reg & D13)
		arg[0]	= AO4MF_AO_BUF_STATUS_1Q_FULL;
	else if (reg & D15)
		arg[0]	= AO4MF_AO_BUF_STATUS_FULL;
	else if (reg & D14)
		arg[0]	= AO4MF_AO_BUF_STATUS_3Q_FULL;
	else
		arg[0]	= AO4MF_AO_BUF_STATUS_MEDIUM;

	return(0);
}



//*****************************************************************************
static int _ao_clk_enable(chan_data_t* chan, s32* arg)
{
	static const s32	options[]	=
	{
		AO4MF_AO_CLK_ENABLE_NO,
		AO4MF_AO_CLK_ENABLE_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(chan->dev, arg, options, chan->vaddr.gsc_bor_32, 5, 5);
	return(ret);
}



//*****************************************************************************
static int _ao_clk_src(chan_data_t* chan, s32* arg)
{
	static const s32	options[]	=
	{
		AO4MF_AO_CLK_SRC_EXT_SW,
		AO4MF_AO_CLK_SRC_INT,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(chan->dev, arg, options, chan->vaddr.gsc_bor_32, 4, 4);
	return(ret);
}



//*****************************************************************************
static int _ao_ext_clking_ready(chan_data_t* chan, s32* arg)
{
	static const s32	options[]	=
	{
		AO4MF_AO_EXT_CLKING_READY_NO,
		AO4MF_AO_EXT_CLKING_READY_YES,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(chan->dev, arg, options, chan->vaddr.gsc_bor_32, 6, 6);
	return(ret);
}



//*****************************************************************************
static int _ao_sw_clk(chan_data_t* chan, s32* arg)
{
	os_reg_mem_mx_u32(chan->dev, chan->vaddr.gsc_bor_32, D7, D7);
	return(0);
}



//*****************************************************************************
static int _burst_mode(chan_data_t* chan, s32* arg)
{
	static const s32	options[]	=
	{
		AO4MF_BURST_MODE_DISABLE,
		AO4MF_BURST_MODE_ENABLE,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(chan->dev, arg, options, chan->vaddr.gsc_bcr_32, 0, 0);
	return(ret);
}



//*****************************************************************************
static int _burst_ready(chan_data_t* chan, s32* arg)
{
	static const s32	options[]	=
	{
		AO4MF_BURST_READY_NO,
		AO4MF_BURST_READY_YES,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(chan->dev, arg, options, chan->vaddr.gsc_bcr_32, 1, 1);
	return(ret);
}



//*****************************************************************************
static int _burst_trigger(chan_data_t* chan, s32* arg)
{
	irq_reg_mod(chan, chan->vaddr.gsc_bcr_32, D2, D2);
	return(0);
}



//*****************************************************************************
static int _data_format(chan_data_t* chan, s32* arg)
{
	static const s32	options[]	=
	{
		AO4MF_DATA_FORMAT_2S_COMP,
		AO4MF_DATA_FORMAT_OFF_BIN,
		-1	// terminate list
	};

	int	ret;

	arg[0]	= -1;
	ret		= gsc_s32_list_reg(chan->dev, arg, options, chan->vaddr.gsc_bcr_32, 4, 4);
	return(ret);
}



//*****************************************************************************
static int _dio_dir(chan_data_t* chan, s32* arg)
{
	static const s32	options[]	=
	{
		AO4MF_DIO_DIR_IN,
		AO4MF_DIO_DIR_OUT,
		-1	// terminate list
	};

	int	ret;

	if (chan->cache.reg_diopcr)
	{
		ret	= gsc_s32_list_reg(chan->dev, arg, options, chan->vaddr.gsc_diopcr_32, 8, 8);
	}
	else
	{
		switch (arg[0])
		{
			default:

				ret	= -EINVAL;
				break;

			case -1:
			case AO4MF_DIO_DIR_IN:
			case AO4MF_DIO_DIR_OUT:

				arg[0]	= -1;
				ret		= 0;
				break;
		}
	}

	return(ret);
}



//*****************************************************************************
static int _dio_rx(chan_data_t* chan, s32* arg)
{
	if (chan->cache.reg_diopcr)
	{
		arg[0]	= os_reg_mem_rx_u32(chan->dev, chan->vaddr.gsc_diopcr_32);
		arg[0]	&= 0xFF;
	}
	else
	{
		arg[0]	= -1;
	}

	return(0);
}



//*****************************************************************************
static int _dio_tx(chan_data_t* chan, s32* arg)
{
	int	ret;

	if ((arg[0] >= 0) && (arg[0] <= 0xFF))
		ret	= 0;
	else
		ret	= -EINVAL;

	if (chan->cache.reg_diopcr == 0)
		arg[0]	= -1;
	else if (ret == 0)
		ret	= gsc_s32_range_reg(chan->dev, arg, 0, 0xFF, chan->vaddr.gsc_diopcr_32, 7, 0);

	return(ret);
}



//*****************************************************************************
static int _fref_clk_src(chan_data_t* chan, s32* arg)
{
	static const s32	options[]	=
	{
		AO4MF_FREF_CLK_SRC_MASTER,
		AO4MF_FREF_CLK_SRC_ADJ,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(chan->dev, arg, options, chan->vaddr.gsc_accr_32, 9, 9);
	return(ret);
}



//*****************************************************************************
static int _int_samp_rate(chan_data_t* chan, s32* arg)
{
	static const s32	options[]	=
	{
		AO4MF_INT_SAMP_RATE_VAR,
		AO4MF_INT_SAMP_RATE_224000_SPS,
		AO4MF_INT_SAMP_RATE_60000_SPS,
		AO4MF_INT_SAMP_RATE_27304_SPS,
		AO4MF_INT_SAMP_RATE_22500_SPS,
		AO4MF_INT_SAMP_RATE_10000_SPS,
		-1	// terminate list
	};

	u32	reg;
	int	ret;

	if (arg[0] == -1)
	{
		ret		= 0;
		reg		= os_reg_mem_rx_u32(chan->dev, chan->vaddr.gsc_icscr_32);
		arg[0]	= reg & 0xF;

		if (arg[0] > AO4MF_INT_SAMP_RATE_10000_SPS)
			arg[0]	= AO4MF_INT_SAMP_RATE_10000_SPS;
	}
	else
	{
		ret	= gsc_s32_list_reg(chan->dev, arg, options, chan->vaddr.gsc_icscr_32, 3, 0);
	}

	return(ret);
}



//*****************************************************************************
static int _nclk(chan_data_t* chan, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(
			chan->dev,
			arg,
			chan->dev->cache.nclk_min,
			chan->dev->cache.nclk_max,
			chan->vaddr.gsc_accr_32,
			8,
			0);
	return(ret);
}



//*****************************************************************************
static int _nrate(chan_data_t* chan, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_reg(
			chan->dev,
			arg,
			chan->dev->cache.nrate_min,
			chan->dev->cache.nrate_max,
			chan->vaddr.gsc_srcr_32,
			15,
			0);
	return(ret);
}



//*****************************************************************************
static int _rate_gen_sel_x(chan_data_t* chan, s32* arg, s32 alt, s32 zero, s32 set, s32 clear)
{
	u32	reg;
	int	ret	= 0;

	switch (arg[0])
	{
		default:

			ret	= -EINVAL;
			break;

		case -1:

			reg		= os_reg_mem_rx_u32(chan->dev, chan->vaddr.gsc_bcr_32);
			arg[0]	= (reg & D7) ? set : clear;
			break;

		case AO4MF_RATE_GEN_SEL_CH_ALT:

			irq_reg_mod(chan, chan->vaddr.gsc_bcr_32, alt, D7);
			break;

		case AO4MF_RATE_GEN_SEL_CH_0:

			irq_reg_mod(chan, chan->vaddr.gsc_bcr_32, zero, D7);
			break;
	}

	return(ret);
}



//*****************************************************************************
static int _rate_gen_sel(chan_data_t* chan, s32* arg)
{
	int	ret;

	if (chan->index == 0)
		ret	= _rate_gen_sel_x(chan, arg, D7, 0, AO4MF_RATE_GEN_SEL_CH_ALT, AO4MF_RATE_GEN_SEL_CH_0);
	else
		ret	= _rate_gen_sel_x(chan, arg, 0, D7, AO4MF_RATE_GEN_SEL_CH_0, AO4MF_RATE_GEN_SEL_CH_ALT);

	return(ret);
}



//*****************************************************************************
static int _tx_io_abort(chan_data_t* chan, s32* arg)
{
	int	ret;

	ret	= gsc_write_abort_active_xfer(chan);

	if (ret < 0)
	{
		arg[0]	= 0;
	}
	else
	{
		ret		= 0;
		arg[0]	= ret;
	}

	return(ret);
}

//*****************************************************************************
static int _tx_io_mode(chan_data_t* chan, void* arg)
{
	static const s32	list[]	=
	{
		GSC_IO_MODE_PIO,
		GSC_IO_MODE_DMA,
		-1
	};

	int	ret;

	ret	= gsc_s32_list_var(arg, list, &chan->tx.io_mode);
	return(ret);
}



//*****************************************************************************
static int _tx_io_timeout(chan_data_t* chan, s32* arg)
{
	int	ret;

	ret	= gsc_s32_range_var(
			arg,
			AO4MF_IO_TIMEOUT_MIN,
			AO4MF_IO_TIMEOUT_INFINITE,
			&chan->tx.timeout_s);
	return(ret);
}



//*****************************************************************************
static int _xcvr_in(chan_data_t* chan, s32* arg)
{
	static const s32	options[]	=
	{
		AO4MF_XCVR_IN_LVDS,
		AO4MF_XCVR_IN_TTL,
		-1	// terminate list
	};

	int	ret;

	ret	= gsc_s32_list_reg(chan->dev, arg, options, chan->vaddr.gsc_bcr_32, 5, 5);
	return(ret);
}



// variables ******************************************************************

const gsc_ioctl_t	dev_ioctl_list[]	=
{
	{ AO4MF_IOCTL_REG_READ,				(void*) _reg_read				},
	{ AO4MF_IOCTL_REG_WRITE,			(void*) _reg_write				},
	{ AO4MF_IOCTL_REG_MOD,				(void*) _reg_mod				},
	{ AO4MF_IOCTL_QUERY,				(void*) _query					},
	{ AO4MF_IOCTL_INITIALIZE,			(void*) initialize_ioctl		},
	{ AO4MF_IOCTL_AUTO_CAL,				(void*) _auto_cal				},
	{ AO4MF_IOCTL_AUTO_CAL_STS,			(void*) _auto_cal_sts			},
	{ AO4MF_IOCTL_AO_BUF_CLEAR,			(void*) _ao_buf_clear			},
	{ AO4MF_IOCTL_AO_BUF_LOAD_READY,	(void*) _ao_buf_load_ready		},
	{ AO4MF_IOCTL_AO_BUF_LOAD_REQ,		(void*) _ao_buf_load_req		},
	{ AO4MF_IOCTL_AO_BUF_MODE,			(void*) _ao_buf_mode			},
	{ AO4MF_IOCTL_AO_BUF_SIZE,			(void*) _ao_buf_size			},
	{ AO4MF_IOCTL_AO_BUF_STATUS,		(void*) _ao_buf_status			},
	{ AO4MF_IOCTL_AO_CLK_ENABLE,		(void*) _ao_clk_enable			},
	{ AO4MF_IOCTL_AO_CLK_SRC,			(void*) _ao_clk_src				},
	{ AO4MF_IOCTL_AO_EXT_CLKING_READY,	(void*) _ao_ext_clking_ready	},
	{ AO4MF_IOCTL_AO_SW_CLK,			(void*) _ao_sw_clk				},
	{ AO4MF_IOCTL_BURST_MODE,			(void*) _burst_mode				},
	{ AO4MF_IOCTL_BURST_READY,			(void*) _burst_ready			},
	{ AO4MF_IOCTL_BURST_TRIGGER,		(void*) _burst_trigger			},
	{ AO4MF_IOCTL_DATA_FORMAT,			(void*) _data_format			},
	{ AO4MF_IOCTL_DIO_DIR,				(void*) _dio_dir				},
	{ AO4MF_IOCTL_DIO_RX,				(void*) _dio_rx					},
	{ AO4MF_IOCTL_DIO_TX,				(void*) _dio_tx					},
	{ AO4MF_IOCTL_FREF_CLK_SRC,			(void*) _fref_clk_src			},
	{ AO4MF_IOCTL_INT_SAMP_RATE,		(void*) _int_samp_rate			},
	{ AO4MF_IOCTL_IRQ_SEL,				(void*) _irq_sel				},
	{ AO4MF_IOCTL_NCLK,					(void*) _nclk					},
	{ AO4MF_IOCTL_NRATE,				(void*) _nrate					},
	{ AO4MF_IOCTL_RATE_GEN_SEL,			(void*) _rate_gen_sel			},
	{ AO4MF_IOCTL_TX_IO_ABORT,			(void*) _tx_io_abort			},
	{ AO4MF_IOCTL_TX_IO_MODE,			(void*) _tx_io_mode				},
	{ AO4MF_IOCTL_TX_IO_TIMEOUT,		(void*) _tx_io_timeout			},
	{ AO4MF_IOCTL_XCVR_IN,				(void*) _xcvr_in				},
	{ AO4MF_IOCTL_WAIT_EVENT,			(void*) gsc_wait_event_ioctl	},
	{ AO4MF_IOCTL_WAIT_CANCEL,			(void*) gsc_wait_cancel_ioctl	},
	{ AO4MF_IOCTL_WAIT_STATUS,			(void*) gsc_wait_status_ioctl	},

	{ -1, NULL }
};


