// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_mbox_devices.c $
// $Rev: 43205 $
// $Date: 2018-07-08 17:01:59 -0500 (Sun, 08 Jul 2018) $

// INtime: Device Driver: source file

#include "main.h"



//*****************************************************************************
static void _service_msg_default(os_msg_t* msg)
{
	if (msg->mbox != BAD_RTHANDLE)
	{
		msg->status	= OS_MSG_STS_MSG_TYPE;
		SendRtData(msg->mbox, &msg, 1);
	}
	else
	{
		db_printf("%s: DEFAULT: Response not sent: invalid mailbox handle\n", DEV_NAME);
	}
}



//*****************************************************************************
static void _service_msg_close(os_service_data_t* ptr, WORD size, os_msg_t* msg)
{
	GSC_ALT_STRUCT_T*	alt	= ptr->alt;

	if ((size == OS_MSG_SIZE_CLOSE) && (msg->size == OS_MSG_SIZE_CLOSE))
	{
		msg->u.close.ret	= os_close(alt);
		msg->status			= OS_MSG_STS_OK;
	}
	else
	{
		msg->u.close.ret	= -EMSGSIZE;
		msg->status			= OS_MSG_STS_MSG_SIZE;
		msg->size			= OS_MSG_SIZE_CLOSE;
		db_printf("%s: CLOSE: Invalid message size: %d\n", DEV_NAME, (int) size);
	}

	if (msg->mbox != BAD_RTHANDLE)
		SendRtData(msg->mbox, msg, msg->size);
}



//*****************************************************************************
static void _service_msg_ioctl_lg(os_service_data_t* ptr, WORD size, os_msg_t* msg)
{
	GSC_ALT_STRUCT_T*	alt	= ptr->alt;
	void*				buf;
	long				bytes;
	void*				vp;

	if ((size == OS_MSG_SIZE_IOCTL_LG) && (msg->size == OS_MSG_SIZE_IOCTL_LG))
	{
		vp	= MapRtSharedMemory((RTHANDLE) msg->u.ioctl_lg.mem);

		if (vp)
		{
			buf					= (void*) ((u32) vp + msg->u.ioctl_lg.offset);
			msg->u.ioctl_lg.ret	= os_ioctl(alt, msg->u.ioctl_lg.cmd, buf);
			msg->status			= OS_MSG_STS_OK;
			FreeRtMemory(vp);
		}
		else
		{
			msg->u.ioctl_lg.ret	= os_get_error_status();
			msg->status			= OS_MSG_STS_MEM_MAP;
			bytes				= size + msg->u.ioctl_lg.offset;
			db_printf(	"%s: IOCTL LG: Failed to map %ld bytes into driver's virtual memory space.\n",
						DEV_NAME,
						(long) bytes);
		}
	}
	else
	{
		msg->u.ioctl_lg.ret	= -EMSGSIZE;
		msg->status			= OS_MSG_STS_MSG_SIZE;
		msg->size			= OS_MSG_SIZE_IOCTL_LG;
		db_printf("%s: IOCTL LG: Invalid message size: %d\n", DEV_NAME, (int) size);
	}

	if (msg->mbox != BAD_RTHANDLE)
		SendRtData(msg->mbox, msg, msg->size);
}



//*****************************************************************************
static void _service_msg_ioctl_sm(os_service_data_t* ptr, WORD size, os_msg_t* msg)
{
	GSC_ALT_STRUCT_T*	alt	= ptr->alt;
	s32					sz;

	if ((size >= OS_MSG_SIZE_IOCTL_SM_MIN)	&&
		(size <= OS_MSG_SIZE_IOCTL_SM_MAX)	&&
		(size == msg->size))
	{
		sz	= OS_IOCTL_SIZE_DECODE(msg->u.ioctl_sm.cmd);

		if (sz == (size - OS_MSG_SIZE_IOCTL_SM_MIN))
		{
			msg->u.ioctl_sm.ret	= os_ioctl(alt, msg->u.ioctl_sm.cmd, msg->u.ioctl_sm.buf);
			msg->status			= OS_MSG_STS_OK;
		}
		else
		{
			msg->u.ioctl_sm.ret	= -EMSGSIZE;
			msg->status			= OS_MSG_STS_MSG_SIZE;
			msg->size			= OS_MSG_SIZE_IOCTL_SM_MIN;
		}
	}
	else
	{
		msg->u.ioctl_sm.ret	= -EMSGSIZE;
		msg->status			= OS_MSG_STS_MSG_SIZE;
		msg->size			= OS_MSG_SIZE_IOCTL_SM_MIN;
		db_printf("%s: IOCTL SM: Invalid message size: %d\n", DEV_NAME, (int) size);
	}

	if (msg->mbox != BAD_RTHANDLE)
		SendRtData(msg->mbox, msg, msg->size);
}



//*****************************************************************************
static void _service_msg_open(os_service_data_t* ptr, WORD size, os_msg_t* msg)
{
	GSC_ALT_STRUCT_T*	alt	= ptr->alt;

	if ((size == OS_MSG_SIZE_OPEN) && (msg->size == OS_MSG_SIZE_OPEN))
	{
		msg->u.open.ret	= os_open(alt, msg->u.open.share);
		msg->status		= OS_MSG_STS_OK;
	}
	else
	{
		msg->u.open.ret	= -EMSGSIZE;
		msg->status		= OS_MSG_STS_MSG_SIZE;
		msg->size		= OS_MSG_SIZE_OPEN;
		db_printf("%s: OPEN: Invalid message size: %d\n", DEV_NAME, (int) size);
	}

	if (msg->mbox != BAD_RTHANDLE)
		SendRtData(msg->mbox, msg, msg->size);
}



//*****************************************************************************
static void _service_msg_read(os_service_data_t* ptr, WORD size, os_msg_t* msg)
{
	GSC_ALT_STRUCT_T*	alt		= ptr->alt;
	void*				buf;
	long				bytes;
	void*				vp;

	if ((size == OS_MSG_SIZE_READ) && (msg->size == OS_MSG_SIZE_READ))
	{
		vp	= MapRtSharedMemory((RTHANDLE) msg->u.read.mem);

		if (vp)
		{
			buf				= (void*) ((u32) vp + msg->u.read.offset);
			msg->u.read.ret	= os_read(alt, buf, msg->u.read.count);
			msg->status		= OS_MSG_STS_OK;
			FreeRtMemory(vp);
		}
		else
		{
			msg->u.read.ret	= os_get_error_status();
			msg->status		= OS_MSG_STS_MEM_MAP;
			bytes			= msg->u.read.count + msg->u.read.offset;
			db_printf(	"%s: READ: Failed to map %ld bytes into driver's virtual memory space.\n",
						DEV_NAME,
						(long) bytes);
		}
	}
	else
	{
		msg->u.read.ret	= -EMSGSIZE;
		msg->status		= OS_MSG_STS_MSG_SIZE;
		msg->size		= OS_MSG_SIZE_READ;
		db_printf("%s: READ: Invalid message size: %d\n", DEV_NAME, (int) size);
	}

	if (msg->mbox != BAD_RTHANDLE)
		SendRtData(msg->mbox, msg, msg->size);
}



//*****************************************************************************
static void _service_msg_write(os_service_data_t* ptr, WORD size, os_msg_t* msg)
{
	GSC_ALT_STRUCT_T*	alt	= ptr->alt;
	void*				buf;
	long				bytes;
	void*				vp;

	if ((size == OS_MSG_SIZE_WRITE) && (msg->size == OS_MSG_SIZE_WRITE))
	{
		vp	= MapRtSharedMemory((RTHANDLE) msg->u.write.mem);

		if (vp)
		{
			buf					= (void*) ((u32) vp + msg->u.write.offset);
			msg->u.write.ret	= os_write(alt, buf, msg->u.write.count);
			msg->status			= OS_MSG_STS_OK;
			FreeRtMemory(vp);
		}
		else
		{
			msg->u.write.ret	= os_get_error_status();
			msg->status			= OS_MSG_STS_MEM_MAP;
			bytes				= msg->u.write.count + msg->u.write.offset;
			db_printf(	"%s: WRITE: Failed to map %ld bytes into driver's virtual memory space.\n",
						DEV_NAME,
						(long) bytes);
		}
	}
	else
	{
		msg->u.write.ret	= -EMSGSIZE;
		msg->status			= OS_MSG_STS_MSG_SIZE;
		msg->size			= OS_MSG_SIZE_WRITE;
		db_printf("%s: WRITE: Invalid message size: %d\n", DEV_NAME, (int) size);
	}

	if (msg->mbox != BAD_RTHANDLE)
		SendRtData(msg->mbox, msg, msg->size);
}



//*****************************************************************************
static void _service_thread(void* arg)
{
	os_msg_t			msg;
	os_service_data_t*	ptr	= (void*) arg;
	WORD				size;

	for (;;)
	{
		ptr->busy	= 0;

		for (;;)	// Get a message.
		{
			size	= ReceiveRtData(ptr->mbox, &msg, 10000);

			if (size)
				break;
		}

		if ((size < 4) || (size != msg.size))
		{
			// The message is too small to process or its content is invalid.
			db_printf("%s: RECEIVE: Invalid message size: %d\n", DEV_NAME, (int) size);

			if (msg.mbox != BAD_RTHANDLE)
			{
				msg.status	= OS_MSG_STS_MSG_SIZE;
				SendRtData(msg.mbox, &msg, 1);
			}

			continue;
		}

		if ((size == OS_MSG_SIZE_EXIT) && (msg.msg_type == OS_MSG_TYPE_EXIT))
			break;

		switch (msg.msg_type)
		{
			default:

				_service_msg_default(&msg);
				continue;

			case OS_MSG_TYPE_CLOSE:

				_service_msg_close(ptr, size, &msg);
				continue;

			case OS_MSG_TYPE_IOCTL_LG:

				_service_msg_ioctl_lg(ptr, size, &msg);
				continue;

			case OS_MSG_TYPE_IOCTL_SM:

				_service_msg_ioctl_sm(ptr, size, &msg);
				continue;

			case OS_MSG_TYPE_OPEN:

				_service_msg_open(ptr, size, &msg);
				continue;

			case OS_MSG_TYPE_READ:

				_service_msg_read(ptr, size, &msg);
				continue;

			case OS_MSG_TYPE_WRITE:

				_service_msg_write(ptr, size, &msg);
				continue;
		}
	}

	DeleteRtMailbox(ptr->mbox);
	memset(ptr, 0, sizeof(ptr[0]));
	ptr->mbox	= BAD_RTHANDLE;
}



//*****************************************************************************
static int _service_thread_start(int index)
{
	RTHANDLE			mbox;
	os_service_data_t*	ptr		= &gsc_global.service_threads.data[index];
	int					ret		= 0;
	RTHANDLE			thread;

	mbox	= CreateRtMailbox(DATA_MAILBOX);

	if (mbox == BAD_RTHANDLE)
	{
		ret	= os_get_error_status();
		db_printf("%s: Failed to create service thread mailbox.\n", DEV_NAME);
	}
	else
	{
		thread	= CreateRtThread(	gsc_global.priority,
									_service_thread,
									4096,
									(void*) ptr);

		if (thread == BAD_RTHANDLE)
		{
			DeleteRtMailbox(mbox);
			ptr->mbox	= BAD_RTHANDLE;
			ret			= os_get_error_status();
			db_printf("%s: Failed to create service thread.\n", DEV_NAME);
		}
		else
		{
			ptr->mbox			= mbox;
			ptr->initialized	= 1;
		}
	}

	return(ret);
}



//*****************************************************************************
static int _service_thread_aquire(void)
{
	int			i;
	int			index	= -1;
	int			lock;
	int			ret;

	lock	= os_sem_lock(&gsc_global.service_threads.sem);

	for (i = 0; i < SIZEOF_ARRAY(gsc_global.service_threads.data); i++)
	{
		if (gsc_global.service_threads.data[i].busy)
			continue;

		if (gsc_global.service_threads.data[i].initialized == 0)
		{
			ret	= _service_thread_start(i);

			if (ret)
			{
				db_printf("%s: SERVICE: Failed to start a service thread %d.\n", DEV_NAME, i);
				continue;
			}
		}

		gsc_global.service_threads.data[i].busy	= 1;
		index	= i;
		break;
	}

	if (lock == 0)
	{
		ret	= os_sem_unlock(&gsc_global.service_threads.sem);

		if (ret)
			db_printf("%s: Failed to unlock service thread .\n", DEV_NAME);
	}

	return(index);
}



//*****************************************************************************
static void _dev_thread(void* arg)
{
	BOOL				cat			= FALSE;
	int					i;
	RTHANDLE			mbox		= BAD_RTHANDLE;
	char				name[64];
	os_msg_t			msg;
	os_dev_thread_t*	ptr			= (void*) arg;
	int					ret;
	RTHANDLE			root;
	WORD				size;

	for (;;)
	{
		ret	= os_get_root_process(&root);

		if (ret)
		{
			db_printf("%s: DEVICE: Failed to access root process.\n", DEV_NAME);
			break;
		}

		mbox	= CreateRtMailbox(DATA_MAILBOX);

		if (mbox == BAD_RTHANDLE)
		{
			ret	= os_get_error_status();
			db_printf(	"%s: DEVICE: Failed to create Device Mailbox: %s\n",
						DEV_NAME,
						name);
			break;
		}

		sprintf(name, "%s.%d", DEV_NAME, ptr->index);
		cat	= CatalogRtHandle(root, mbox, name);

		if (cat == FALSE)
		{
			ret	= os_get_error_status();
			db_printf(	"%s: DEVICE: Failed to catalog Device Mailbox: %s\n",
						DEV_NAME,
						name);
			break;
		}

		// Start processing messages.
		ptr->mbox		= mbox;
		ptr->started	= 1;

		for (;;)	// Process all messages.
		{
			for (;;)	// Get a message.
			{
				size	= ReceiveRtData(mbox, &msg, 10000);

				if (size)
					break;
			}

			if ((size == OS_MSG_SIZE_EXIT) && (msg.msg_type == OS_MSG_TYPE_EXIT))
				break;

			i	= _service_thread_aquire();

			if (i < 0)
			{
				msg.status	= OS_MSG_STS_THREAD;

				if (msg.mbox != BAD_RTHANDLE)
					SendRtData(msg.mbox, &msg, 1);

				continue;
			}

			gsc_global.service_threads.data[i].alt	= ptr->alt;
			SendRtData(gsc_global.service_threads.data[i].mbox, &msg, size);
		}

		break;
	}

	if (cat)
		UncatalogRtHandle(root, name);

	if (mbox != BAD_RTHANDLE)
		DeleteRtMailbox(mbox);

	// Wait til we're told we can exit.

	for (i = 0; i < 1000; i++)
	{
		if (ptr->exit)
			break;

		os_time_sleep_ms(10);
	}

	memset(ptr, 0, sizeof(ptr[0]));
}



//*****************************************************************************
static int _dev_thread_start(GSC_ALT_STRUCT_T* alt, int index)
{
	int					i;
	os_dev_thread_t*	ptr;
	int					ret;
	RTHANDLE			thread;

	for (;;)
	{
		ptr				= &gsc_global.device_threads[index];
		ptr->alt		= alt;
		ptr->ret		= 0;
		ptr->started	= 0;
		ptr->index		= index;
		thread			= CreateRtThread(	gsc_global.priority,
											_dev_thread,
											4096,
											(void*) ptr);

		if (thread == BAD_RTHANDLE)
		{
			ret	= os_get_error_status();
			db_printf("%s: Failed create device thread for device %d.", DEV_NAME, index);
			break;
		}

		// Wait for thread to signal its status.

		for (i = 0; i < 1000; i++)
		{
			if (ptr->started)
				break;

			if (ptr->ret)
				break;

			os_time_sleep_ms(10);
		}

		ret			= ptr->ret;
		ptr->exit	= 1;			// Tell the thread they can exit when the time comes.
		break;
	}

	return(ret);
}



//*****************************************************************************
static int _dev_thread_acquire(GSC_ALT_STRUCT_T* alt, int index)
{
	int	i;
	int	ret		= -ENOMEM;

	for (i = 0; i < SIZEOF_ARRAY(gsc_global.device_threads); i++)
	{
		if (gsc_global.device_threads[i].in_use)
			continue;

		ret	= _dev_thread_start(alt, i);

		if (ret)
			continue;

		gsc_global.device_threads[i].in_use	= 1;
		ret	= 0;
		break;
	}

	return(ret);
}



#if (GSC_DEVS_PER_BOARD > 1)

//*****************************************************************************
int os_mbox_devices_start(void)
{
	int	i;
	int	j;
	int	index	= 0;
	int	lock;
	int	ret		= 0;

	lock	= os_sem_lock(&gsc_global.sem);

	os_sem_create(&gsc_global.service_threads.sem);	// destroyed on driver exit.

	for (i = 0; i < SIZEOF_ARRAY(gsc_global.dev_list); i++)
	{
		if (gsc_global.dev_list[i] == NULL)
			continue;

		for (j = 0; (ret == 0) && (j < GSC_DEVS_PER_BOARD); j++)
		{
			ret	= _dev_thread_acquire(&gsc_global.dev_list[i]->channel[j], index);
			index++;
		}
	}

	if (lock == 0)
		os_sem_unlock(&gsc_global.sem);

	return(ret);
}

#else

//*****************************************************************************
int os_mbox_devices_start(void)
{
	int	i;
	int	index	= 0;
	int	lock;
	int	ret		= 0;

	lock	= os_sem_lock(&gsc_global.sem);

	os_sem_create(&gsc_global.service_threads.sem);	// destroyed on driver exit.

	for (i = 0; (ret == 0) && (i < SIZEOF_ARRAY(gsc_global.dev_list)); i++)
	{
		if (gsc_global.dev_list[i] == NULL)
			continue;

		ret	= _dev_thread_acquire(gsc_global.dev_list[i], index);
		index++;
	}

	if (lock == 0)
		os_sem_unlock(&gsc_global.sem);

	return(ret);
}

#endif


