// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_mbox_driver.c $
// $Rev: 43205 $
// $Date: 2018-07-08 17:01:59 -0500 (Sun, 08 Jul 2018) $

// INtime: Device Driver: source file

#include "main.h"



// #define ********************************************************************

#define	PREFIX		DEV_NAME ": Driver Mailbox"



//*****************************************************************************
// Append the given string to the mesasage buffer, if there is space.
static void _append_to_buffer(const char* src)
{
	size_t	len_buf;
	size_t	len_src;

	for (;;)	// A convenience loop.
	{
		if (src == NULL)
		{
			db_printf("%s: Internal error: %s, %d\n", DEV_NAME, __FILE__, __LINE__);
			break;
		}

		len_buf	= strlen(gsc_global.driver.buffer);
		len_src	= strlen(src);

		if ((len_buf + len_src) >= (sizeof(gsc_global.driver.buffer)))
		{
			db_printf("%s: Buffer too small - content truncated.\n", PREFIX);
			break;
		}

		strcat(gsc_global.driver.buffer, src);
		break;
	}
}



//*****************************************************************************
// Return:
//	< 0	Never returned
//	>=0	Length of buffer string after operation
static int _fill_buffer(void)
{
	char		buf[32];
	const char*	comma;
    int			i;

	fflush(stdout);

	// Initialize the buiffer.
	gsc_global.driver.buffer[0]	= 0;

	// Add content.
	_append_to_buffer("version: " GSC_DRIVER_VERSION "\n");

	sprintf(buf, "boards: %d\n", (int) gsc_global.dev_qty);
	_append_to_buffer(buf);

	_append_to_buffer("models: ");
	comma	= "";

	for (i = 0; i < SIZEOF_ARRAY(gsc_global.dev_list); i++)
	{
		if (gsc_global.dev_list[i])
		{
			_append_to_buffer(comma);
			_append_to_buffer(gsc_global.dev_list[i]->model);
			comma	= ",";
		}
	}

	_append_to_buffer("\n");

#if defined(DEV_SUPPORTS_PROC_ID_STR)

	// This is intended to accommodate board identification
	// for those that have user id jumpers.
	_append_to_buffer("ids: ");
	comma	= "";

	for (i = 0; i < SIZEOF_ARRAY(gsc_global.dev_list); i++)
	{
		if (gsc_global.dev_list[i])
		{
			_append_to_buffer(comma);
			_append_to_buffer(gsc_global.dev_list[i]->proc_id_str);
			comma	= ",";
		}
	}

	_append_to_buffer("\n");

#endif

	i	= strlen(gsc_global.driver.buffer);
	fflush(stdout);
    return(i);
}



//*****************************************************************************
// Return:
//	< 0	Never returned
//	>=0	Volume of data copied to destination.
static int _read_data(char* dst, int size)
{
	int	len;

	len	= _fill_buffer();

	if (size == 0)
	{
		len	= 0;
	}
	else if ((len  + 1) <= size)
	{
		// Copy the entire message content.
		strcpy(dst, gsc_global.driver.buffer);
	}
	else
	{
		// Copy a portion of the message content.
		len	= size - 1;
		strncpy(dst, gsc_global.driver.buffer, size);
		dst[len]	= 0;
	}

	return(len);
}



//*****************************************************************************
static void _service_msg_close(WORD size, os_msg_t* msg)
{
	BOOLEAN	sts;

	if ((size == OS_MSG_SIZE_CLOSE) && (msg->size == OS_MSG_SIZE_CLOSE))
	{
		msg->status			= OS_MSG_STS_OK;
		msg->u.close.ret	= 0;
	}
	else
	{
		msg->status	= OS_MSG_STS_MSG_SIZE;
		msg->size	= 1;
	}

	if (msg->mbox != BAD_RTHANDLE)
	{
		sts	= SendRtData(msg->mbox, msg, msg->size);

		if (sts == FALSE)
		{
			os_get_error_status();
			db_printf("%s: Failed to send CLOSE response.\n", PREFIX);
		}
	}
}



//*****************************************************************************
static void _service_msg_default(os_msg_t* msg)
{
	BOOLEAN	sts;

	if (msg->mbox != BAD_RTHANDLE)
	{
		msg->status	= OS_MSG_STS_MSG_TYPE;
		sts	= SendRtData(msg->mbox, &msg, 1);

		if (sts == FALSE)
		{
			os_get_error_status();
			db_printf("%s: Failed to send default response.\n", PREFIX);
		}
	}
}



//*****************************************************************************
static void _service_msg_open(WORD size, os_msg_t* msg)
{
	BOOLEAN	sts;

	if ((size == OS_MSG_SIZE_OPEN) && (msg->size == OS_MSG_SIZE_OPEN))
	{
		msg->status		= OS_MSG_STS_OK;
		msg->u.open.ret	= 0;
	}
	else
	{
		msg->status	= OS_MSG_STS_MSG_SIZE;
		msg->size	= 1;
	}

	if (msg->mbox != BAD_RTHANDLE)
	{
		sts	= SendRtData(msg->mbox, msg, msg->size);

		if (sts == FALSE)
		{
			os_get_error_status();
			db_printf("%s: Failed to send OPEN response.\n", PREFIX);
		}
	}
}



//*****************************************************************************
static void _service_msg_read(WORD size, os_msg_t* msg)
{
	void*	ptr;
	int		ret;
	BOOLEAN	sts;
	int		unlock	= 0;
	void*	vp;

	for (;;)	// A convenience loop.
	{
		// Validate the message size.

		if ((size != OS_MSG_SIZE_READ) || (msg->size != OS_MSG_SIZE_READ))
		{
			msg->status	= OS_MSG_STS_MSG_SIZE;
			msg->size	= 1;
			db_printf(	"%s: Received invalid READ message size (%d bytes).\n",
						PREFIX,
						(int) size);
			break;
		}

		// Get the global lock so the device list remains stable while we work.
		ret	= os_sem_lock(&gsc_global.sem);

		if (ret)
		{
			msg->status	= OS_MSG_STS_LOCK;
			msg->size	= 1;
			db_printf("%s: Failed to acquire master semaphore.\n", PREFIX);
			break;
		}

		unlock	= 1;

		// Access the application's read buffer.
		vp	= MapRtSharedMemory((RTHANDLE) msg->u.read.mem);

		if (vp == NULL)
		{
			msg->status	= OS_MSG_STS_MEM_MAP;
			msg->size	= 1;
			db_printf("%s: Failed to map READ memory.\n", PREFIX);
			break;
		}

		// Generate the content and copy it to the application buffer.
		ptr				= (void*) ((u32) vp + msg->u.read.offset);
		msg->status		= OS_MSG_STS_OK;
		msg->u.read.ret	= _read_data(ptr, msg->u.read.count);
		ret				= (int) FreeRtMemory(vp);

		if (ret == -1)
		{
			os_get_error_status();
			db_printf("%s: Failed to unmap READ memory.\n", PREFIX);
		}

		break;
	}

	// Release the master lack as the work is done.

	if (unlock)
	{
		ret	= os_sem_unlock(&gsc_global.sem);

		if (ret)
			db_printf("%s: Failed to release master semaphore.\n", PREFIX);
	}

	// Send the response.

	if (msg->mbox != BAD_RTHANDLE)
	{
		sts	= SendRtData(msg->mbox, msg, msg->size);

		if (sts == FALSE)
		{
			ret	= os_get_error_status();
			db_printf("%s: Failed to send READ response.\n", PREFIX);
		}
	}
}



//*****************************************************************************
//	0	Success
//	< 0	Failure: value is -errno or -GetLastRtError()
//	> 0	Never returned
static int _mbox_create(void)
{
	BOOL		cat;
	int			ret;
	RTHANDLE	root;

	for (;;)
	{
		ret	= os_get_root_process(&root);

		if (ret)
		{
			gsc_global.driver.failed	= __LINE__;
			break;
		}

		gsc_global.driver.mbox	= CreateRtMailbox(DATA_MAILBOX);

		if (gsc_global.driver.mbox == BAD_RTHANDLE)
		{
			ret	= os_get_error_status();
			db_printf("%s: Failed to create Driver Mailbox.\n", PREFIX);
			gsc_global.driver.failed	= __LINE__;
			break;
		}

		cat	= CatalogRtHandle(root, gsc_global.driver.mbox, DEV_NAME);

		if (cat == FALSE)
		{
			ret	= os_get_error_status();
			db_printf("%s: Failed to catalogc Driver Mailbox.\n", PREFIX);
			DeleteRtMailbox(gsc_global.driver.mbox);
			gsc_global.driver.mbox	= BAD_RTHANDLE;
			gsc_global.driver.failed	= __LINE__;
			break;
		}

		ret	= 0;
		break;
	}

	return(ret);
}



//*****************************************************************************
static void _mbox_destroy(void)
{
	int			ret;
	RTHANDLE	root;

	ret	= os_get_root_process(&root);

	if (ret == 0)
	{
		UncatalogRtHandle(root, DEV_NAME);

		if (gsc_global.driver.mbox != BAD_RTHANDLE)
			DeleteRtMailbox(gsc_global.driver.mbox);
	}
}



//*****************************************************************************
static void _mbox_service(void)
{
	int			lock	= -1;
	os_msg_t	msg;
	WORD		size;
	RTHANDLE	thread;
	BOOLEAN		sts;

	for (;;)	// A convenience loop.
	{
		// Get our thread handle.
		thread	= GetRtThreadHandles(THIS_THREAD);

		if (thread == BAD_RTHANDLE)
		{
			db_printf("%s: Failed to get thread handle.", PREFIX);
			gsc_global.driver.failed	= 1;
			gsc_global.driver.failed	= __LINE__;
			break;
		}

		sts	= SetRtThreadPriority(thread, gsc_global.priority);

		if (sts == FALSE)
		{
			// The thread priority could not be adjusted.
			// We'll report it and continue on.
			db_printf("%s: Failed to adjust DRIVER thread priority.\n", PREFIX);
			db_printf("%s: Priority requested: %d\n", PREFIX,  (int) gsc_global.priority);
		}

		// Begin processing messages.
		gsc_global.driver.started	= 1;

		for (;;)
		{
			for (;;)
			{
				size	= ReceiveRtData(gsc_global.driver.mbox, &msg, 5000);

				if (size)
					break;
			}

			// Verify that the message size is valid.

			if ((size < 4) || (size != msg.size))
			{
				// The message is too small to process or its content is invalid.
				db_printf(	"%s: Received message with invalid size (%d bytes).\n",
							PREFIX,
							(int) size);

				if (msg.mbox != BAD_RTHANDLE)
				{
					msg.status	= OS_MSG_STS_MSG_SIZE;
					sts			= SendRtData(msg.mbox, &msg, 1);

					if (sts == FALSE)
					{
						os_get_error_status();
						db_printf("%s: Failed to send error response.\n", PREFIX);
					}
				}

				continue;
			}

			if ((size == OS_MSG_SIZE_EXIT) && (msg.msg_type == OS_MSG_TYPE_EXIT))
			{
				// We received an exit message.
				gsc_global.driver.exit_msg	= 1;
				break;
			}

			switch (msg.msg_type)
			{
				default:

					_service_msg_default(&msg);
					continue;

				case OS_MSG_TYPE_CLOSE:

					_service_msg_close(size, &msg);
					continue;

				case OS_MSG_TYPE_OPEN:

					_service_msg_open(size, &msg);
					continue;

				case OS_MSG_TYPE_READ:

					_service_msg_read(size, &msg);
					continue;
			}
		}

		break;
	}

	gsc_global.driver.stopped	= 1;
}



//*****************************************************************************
void os_mbox_driver_start(void)
{
	int	ret;

	ret	= _mbox_create();

	if (ret == 0)
		_mbox_service();

	_mbox_destroy();
}



//*****************************************************************************
// Returns: 0 = driver not loaded, else driver is loaded
int os_mbox_driver_exists(void)
{
	int			loaded	= 0;
	RTHANDLE	mbox;
	int			ret;
	RTHANDLE	root;

	for (;;)
	{
		ret	= os_get_root_process(&root);

		if (ret)
			break;

		mbox	= LookupRtHandle(root, DEV_NAME, NO_WAIT);

		if (mbox == BAD_RTHANDLE)
			break;

		loaded	= 1;
		printf("%s: The driver is already running.\n", DEV_NAME);
		break;
	}

	return(loaded);
}


