// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_pci.c $
// $Rev: 43205 $
// $Date: 2018-07-08 17:01:59 -0500 (Sun, 08 Jul 2018) $

// INtime: Device Driver: source file

#include "main.h"



//*****************************************************************************
// Return:
//	0	Success
//	< 0	Failure: value is -errno or -GetLastRtError()
//	> 0	Never returned
int os_pci_dev_enable(os_pci_t* pci)
{
	int		ret;
	BOOLEAN	sts;

	if (pci == NULL)
	{
		ret	= -EFAULT;
	}
	else if (pci->pcidev.bUnusable)
	{
		// The device hasn't been passed to INtime.
		ret	= -EIO;
		printf("%s: Device not passed to INtime.\n", DEV_NAME);
	}
	else if (pci->enabled)
	{
		// There is nothing to do.
		ret	= 0;
	}
	else
	{
		sts	= PciEnableDevice(&pci->pcidev);

		if (sts == FALSE)
		{
			ret	= -EIO;
			db_printf("%s: Failed to enable PCI device\n", DEV_NAME);
		}
		else
		{
			pci->enabled	= 1;
			ret				= 0;
		}
	}

	return(ret);
}



//*****************************************************************************
void os_pci_dev_disable(os_pci_t* pci)
{
	if ((pci) && (pci->enabled))
	{
		// Ordinarily we would call PciDisableDevice(). However, in this call
		// INtime will power down the board and perform a soft PCI reset. This
		// will cause the PLX chip to loose its resource settings. Therefore,
		// this call is never made.
		pci->enabled	= 0;
	}
}



//*****************************************************************************
void os_pci_master_clear(os_pci_t* pci)
{
	// INtime has no such service.
}



//*****************************************************************************
int os_pci_master_set(os_pci_t* pci)
{
	if ((pci) && (pci->enabled))
		PciSetMaster(&pci->pcidev);

	return(0);
}


