// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_sem.c $
// $Rev: 43205 $
// $Date: 2018-07-08 17:01:59 -0500 (Sun, 08 Jul 2018) $

// INtime: Device Driver: source file

#include "main.h"



//*****************************************************************************
// Return:
//	0	Success
//	< 0	Failure: value is -errno or -GetLastRtError()
//	> 0	Never returned
int os_sem_create(os_sem_t* sem)
{
	int	ret	= 0;

	for (;;)	// convenience loop
	{
		if (sem == NULL)
		{
			ret	= -EINVAL;
			break;
		}

		memset(sem, 0, sizeof(os_sem_t));
		sem->sem	= CreateRtSemaphore(1, 1, PRIORITY_QUEUING);

		if (sem->sem == BAD_RTHANDLE)
		{
			ret	= os_get_error_status();
			db_printf("%s: Failed to create semaphore.\n", DEV_NAME);
			memset(sem, 0, sizeof(os_sem_t));
		}

		break;
	}

	return(ret);
}



//*****************************************************************************
// Return:
//	0	Success
//	< 0	Failure: value is -errno or -GetLastRtError()
//	> 0	Never returned
int os_sem_create_qty(int max_qty, int start_qty, os_sem_t* sem)
{
	int	ret	= 0;

	for (;;)	// convenience loop
	{
		if ((max_qty < 0) || (max_qty > 65535) ||
			(start_qty < 0) || (start_qty > max_qty) ||
			(sem == NULL))
		{
			ret	= -EINVAL;
			break;
		}

		memset(sem, 0, sizeof(os_sem_t));
		sem->sem	= CreateRtSemaphore(start_qty, max_qty, PRIORITY_QUEUING);

		if (sem->sem == BAD_RTHANDLE)
		{
			ret	= os_get_error_status();
			db_printf("%s: Failed to create semaphore.\n", DEV_NAME);
			memset(sem, 0, sizeof(os_sem_t));
		}

		break;
	}

	return(ret);
}



//*****************************************************************************
// Return:
//	0	Success
//	< 0	Failure: value is -errno or -GetLastRtError()
//	> 0	Never returned
int os_sem_destroy(os_sem_t* sem)
{
	int		ret		= 0;
	BOOLEAN	sts;
	
	for (;;)	// convenience loop
	{
		if (sem == NULL)
		{
			ret	= -EINVAL;
			break;
		}

		if (sem->sem == 0)
		{
			ret	= -EINVAL;
			memset(sem, 0, sizeof(os_sem_t));
			break;
		}

		sts	= DeleteRtSemaphore(sem->sem);

		if (sts == FALSE)
		{
			ret	= os_get_error_status();
			db_printf("%s: Failed to delete semaphore.\n", DEV_NAME);
		}

		memset(sem, 0, sizeof(os_sem_t));
		break;
	}

	return(ret);
}



//*****************************************************************************
// Return:
//	0	Success
//	< 0	Failure: value is -errno or -GetLastRtError()
//	> 0	Never returned
int os_sem_lock(os_sem_t* sem)
{
	int		ret;
	DWORD	sts;

	for (;;)	// convenience loop
	{
		if (sem == NULL)
		{
			ret	= -EINVAL;
			break;
		}

		if (sem->sem == 0)
		{
			ret	= -EINVAL;
			break;
		}

		sts	= WaitForRtSemaphore(sem->sem, 1, WAIT_FOREVER);

		if (sts == WAIT_FAILED)
		{
			ret	= os_get_error_status();
			db_printf("%s: Failed to wait for semaphore.\n", DEV_NAME);
			break;
		}

		ret	= 0;
		break;
	}
 
	return(ret);
}



//*****************************************************************************
// Return:
//	0	Success
//	< 0	Failure: value is -errno or -GetLastRtError()
//	> 0	Never returned
int os_sem_unlock(os_sem_t* sem)
{
	int		ret;
	BOOLEAN	sts;

	for (;;)	// convenience loop
	{
		if (sem == NULL)
		{
			ret	= -EINVAL;
			break;
		}

		if (sem->sem == 0)
		{
			ret	= -EINVAL;
			break;
		}

		sts	= ReleaseRtSemaphore(sem->sem, 1);

		if (sts == FALSE)
		{
			ret	= os_get_error_status();
			db_printf("%s: Failed to release semaphore.\n", DEV_NAME);
			break;
		}

		ret	= 0;
		break;
	}

	return(ret);
}


