// $URL: http://subversion:8080/svn/gsc/trunk/drivers/gsc_common/driver/intime/os_time.c $
// $Rev: 43205 $
// $Date: 2018-07-08 17:01:59 -0500 (Sun, 08 Jul 2018) $

// INtime: Device Driver: source file

#include "main.h"



//*****************************************************************************
long os_time_delta_ms(const os_time_t* tt1, const os_time_t* tt2)
{
	long	ms;
	QWORD	ticks;
	QWORD	us;

	if (tt1 && tt2)
	{
		ticks	= tt2->count - tt1->count;
		us		= (ticks * gsc_global.tick_us) + (gsc_global.tick_us - 1);
		ms		= (long) (us / 1000);
	}
	else
	{
		ms	= 0;
	}

	return(ms);
}



//*****************************************************************************
void os_time_get(os_time_t* tt)
{
	if (tt)
		tt->count	= knGetKernelTime(NULL);
}



//*****************************************************************************
os_time_tick_t os_time_ms_to_ticks(long ms)
{
	os_time_tick_t	ticks;
	long long		us;

	us		= (ms * 1000) + (gsc_global.tick_us - 1);
	ticks	= us / gsc_global.tick_us;
	return(ticks);
}



//*****************************************************************************
os_time_tick_t os_time_tick_get(void)
{
	os_time_tick_t	ticks;

	ticks	= knGetKernelTime(NULL);
	return(ticks);
}



//*****************************************************************************
int os_time_tick_rate(void)
{
	return(gsc_global.tick_rate);
}



//*****************************************************************************
int os_time_tick_sleep(int ticks)
{
	long	ms;
	int		ret;
	BOOLEAN	sts;

	ms	= ((gsc_global.tick_us * ticks) / 1000) + 1;
	sts	= RtSleepEx(ms);

	if (sts == FALSE)
	{
		ret	= os_get_error_status();
		db_printf("%s: Failed to sleep for %ld ticks.\n", DEV_NAME, (long) ticks);
	}
	else
	{
		ret	= 0;
	}

	return(ret);
}



//*****************************************************************************
// Return:
//	0	timeout has not occurred
//	1	timeout has occurred
int os_time_tick_timedout(os_time_tick_t end_time)
{
	QWORD	ticks;
	int		timedout;

	ticks		= knGetKernelTime(NULL);
	timedout	= (end_time <= ticks) ? 1 : 0;
	return(timedout);
}



//*****************************************************************************
long os_time_ticks_to_ms(os_time_tick_t ticks)
{
	long		ms;
	long long	us;

	us	= ticks * gsc_global.tick_us;
	ms	= (long) (us / 1000);
	return(ms);
}



//*****************************************************************************
void os_time_us_delay(long us)
{
	long	ms;

	// INtime doesn't have any delay capabilities.
	ms	= (us + 999) / 1000;

	if (ms < 1)
		ms	= 1;

	RtSleepEx(ms);
}



//*****************************************************************************
void os_time_sleep_ms(long ms)
{
	RtSleepEx(ms);
}


