// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/samples/din/dio.c $
// $Rev: 43280 $
// $Date: 2018-07-09 17:41:35 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Sample Application: source file

#include "main.h"



/******************************************************************************
*
*	Function:	_bin_show
*
*	Purpose:
*
*		Display the given data in binary.
*
*	Arguments:
*
*		data	The data to show.
*
*	Returned:
*
*		None.
*
******************************************************************************/

static void _bin_show(u8 data)
{
	u8	bit;

	for (bit = 0x80; bit; bit /= 2)
	{
		if (bit & data)
			printf("1");
		else
			printf("0");
	}

	fflush(stdout);
}



/******************************************************************************
*
*	Function:	_dev_init
*
*	Purpose:
*
*		Initialize the board.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _dev_init(int fd)
{
	int	errs;
	int	ret;

	ret		= ao4mf_ioctl(fd, AO4MF_IOCTL_INITIALIZE, NULL);
	errs	= ret ? 1 : 0;
	return(errs);
}



/******************************************************************************
*
*	Function:	_dev_dio_port_dir_set_in
*
*	Purpose:
*
*		Set the port to an input.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _dev_dio_port_dir_set_in(int fd)
{
	int	errs;
	int	ret;
	s32	val		= AO4MF_DIO_DIR_IN;

	ret		= ao4mf_ioctl(fd, AO4MF_IOCTL_DIO_DIR, &val);
	errs	= ret ? 1 : 0;
	return(errs);
}



/******************************************************************************
*
*	Function:	_dev_dio_port_in
*
*	Purpose:
*
*		Read from the Digital I/O Port input.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _dev_dio_port_in(int fd)
{
	u8	data;
	int	errs;
	int	ret;
	u32	val;

	ret		= ao4mf_ioctl(fd, AO4MF_IOCTL_DIO_RX, &val);
	errs	= ret ? 1 : 0;
	data	= val;
	_bin_show(data);
	return(errs);
}



/******************************************************************************
*
*	Function:	dio_in
*
*	Purpose:
*
*		Perform a Dogital Input test.
*
*	Arguments:
*
*		fd		The file descriptor for the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int dio_in(int fd)
{
	int	errs	= 0;
	int	i;
	int	ret;
	s32	val;

	for (;;)	// a CONVENIENCE LOOP.
	{
		gsc_label("Digital Input");
		val	= AO4MF_QUERY_REG_DIOPCR;
		ret	= ao4mf_ioctl(fd, AO4MF_IOCTL_QUERY, &val);

		if (ret)
		{
			errs++;
			printf("FAIL <---  (%d. ERROR: ao4mf_ioctl())\n", __LINE__);
			break;
		}

		if (val == 0)
		{
			printf("SKIPPED  (This channel doesn't support digital I/O.)\n");
			break;
		}

		errs	+= _dev_init(fd);
		errs	+= _dev_dio_port_dir_set_in(fd);

		if (errs)
		{
			printf("FAIL <---\n");
			break;
		}

		printf("\n");
		printf("    DIO PORT\n");
		printf("    76543210\n");
		printf("    ========\n");

		for (i = 0; i < 16; i++)
		{
			printf("    ");
			errs	+= _dev_dio_port_in(fd);
			printf("\n");
			gsc_time_sleep_ms(250);
		}

		gsc_label_level_inc();
		errs	+= _dev_init(fd);
		gsc_label("Status");
		printf("%s\n", errs ? "FAIL <---" : "PASS");
		gsc_label_level_dec();
		break;
	}

	return(errs);
}



