// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/samples/irq/service.c $
// $Rev: 43284 $
// $Date: 2018-07-09 17:42:34 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Sample Application: source file

#include "main.h"



// #defines *******************************************************************

#define _30_SEC		(30L * 1000L)



// variables *******************************************************************

static	os_sem_t	sem_enter;
static	os_sem_t	sem_exit;
static	gsc_wait_t	wait_dst;
static	int			wait_fd;



//*****************************************************************************
static int _wait_thread(void* arg)
{
	int	ret;

	os_sem_unlock(&sem_enter);
	ret	= ao4mf_ioctl(wait_fd, AO4MF_IOCTL_WAIT_EVENT, &wait_dst);

	if (ret)
		wait_dst.flags	= 0xFFFFFFFF;

	os_sem_unlock(&sem_exit);
	return(0);
}



// variables *******************************************************************

struct _wait_src_t
{
	int			(*function)(int fd);
	gsc_wait_t	wait;
};



//*****************************************************************************
static int _process_thread(int fd, const struct _wait_src_t* src)
{
	char		buf[64];
	int			errs	= 0;
	int			i;
	int			ret;
	os_thread_t	thread;
	gsc_wait_t	wait;

	errs	+= ao4mf_initialize(fd, -1, 0);

	for (;;)	// A convenience loop.
	{
		os_sem_create_qty(&sem_enter, 1, 0);
		os_sem_create_qty(&sem_exit, 1, 0);
		memset(&thread, 0, sizeof(thread));
		sprintf(buf, "wait event");
		wait_dst	= src->wait;
		errs		+= os_thread_create(&thread, buf, _wait_thread, NULL);

		// Wait for the thread to become waiting.
		os_sem_lock(&sem_enter);

		if (errs == 0)
		{
			wait	= src->wait;

			for (i = 0; i < 100; i++)
			{
				ret	= ao4mf_ioctl(wait_fd, AO4MF_IOCTL_WAIT_STATUS, &wait);

				if (ret)
				{
					printf(	"FAIL <---  (%d. status request: %d)\n",
							__LINE__,
							ret);
					errs++;
					break;
				}

				if (wait.count == 0)
				{
					gsc_time_sleep_ms(100);
					continue;
				}

				if (wait.count == 1)
					break;

				errs++;
				printf(	"FAIL <---  (%d. invalid wait count %ld)\n",
						__LINE__,
						(long) wait.count);
				break;
			}
		}

		// Initiate the respective event.
		errs	+= src->function(wait_fd);

		if (errs)
			break;		// There was an error.

		// Wait for the resumed thread to run.
		os_sem_lock(&sem_exit);

		for (i = 0;; i++)
		{
			if (wait_dst.flags)
				break;

			if (i >= 100)
			{
				printf(	"FAIL <---  (%d. thread failed to resume)\n",
						__LINE__);
				errs++;
				break;
			}

			gsc_time_sleep_ms(100);
		}

		// Verify that the wait ended as expected.

		if (src->wait.flags == 0xFFFFFFFF)
			break;		// There was an error.

		if (errs)
			break;		// There was an error.

		if (src->wait.timeout_ms == _30_SEC)
		{
			if (wait_dst.flags != GSC_WAIT_FLAG_DONE)
			{
				printf(	"FAIL <---  (%d. flag: expect 0x%lX, got 0x%lX)\n",
						__LINE__,
						(long) GSC_WAIT_FLAG_DONE,
						(long) wait_dst.flags);
				errs++;
			}

			break;
		}

		printf("FAIL <---  (%d. INTERNAL ERROR)\n", __LINE__);
		errs++;
		break;
	}

	// Force termination, just in case of an error.
	wait.flags			= 0;
	wait.main			= 0xFFFFFFFF;
	wait.gsc			= 0xFFFFFFFF;
	wait.alt			= 0xFFFFFFFF;
	wait.io				= 0xFFFFFFFF;
	wait.timeout_ms		= 0;
	wait.count			= 0;
	ret		= ao4mf_ioctl(wait_fd, AO4MF_IOCTL_WAIT_CANCEL, &wait);
	errs	+= ret ? 1 : 0;
	errs	+= os_thread_destroy(&thread);

	errs	+= ao4mf_initialize(fd, -1, 0);

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}



//*****************************************************************************
int irq_wait_test(int fd, u32 gsc, int (*callback)(int fd))
{
	int					errs	= 0;
	struct _wait_src_t	src;

	wait_fd	= fd;
	memset(&wait_dst, 0, sizeof(wait_dst));
	memset(&src, 0, sizeof(src));
	src.function		= callback;
	src.wait.gsc		= gsc;
	src.wait.timeout_ms	= _30_SEC;
	errs	+= _process_thread(fd, &src);
	return(errs);
}


