// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/samples/regs/main.c $
// $Rev: 43285 $
// $Date: 2018-07-09 17:42:48 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Sample Application: source file

#include "main.h"



// variables ******************************************************************

static	int	_def_index	= 0;



//*****************************************************************************
static int _parse_args(int argc, char** argv)
{
	char	buf[64];
	char	c;
	int		errs	= 0;
	int		i;
	int		j;
	long	k;

	printf("regs - Register Access Application\n");
	printf("USAGE: regs <index>\n");
	printf("  index  The zero based index of the device to access.\n");

	gsc_label("Arguments");
	printf("(%d argument%s)\n", argc - 1, ((argc - 1) == 1) ? "s" : "");
	gsc_label_level_inc();

	for (i = 0; i < argc; i++)
	{
		sprintf(buf, "Argument %d", i);
		gsc_label(buf);
		printf("%s\n", argv[i]);
	}

	gsc_label_level_dec();

	for (i = 1; i < argc; i++)
	{
		j	= sscanf(argv[i], "%ld%c", &k, &c);

		if ((j == 1) && (k >= 0))
		{
			_def_index	= k;
			continue;
		}

		errs	= 1;
		printf("ERROR: invalid argument: #%d = '%s'\n", i, argv[i]);
		break;
	}

	return(errs);
}



//*****************************************************************************
static int _perform_tests(int fd)
{
	int	errs	= 0;

	main_menu(fd);

	return(errs);
}



//*****************************************************************************
static void _show_access_index(int index)
{
	gsc_label("Accessing Device");
	printf("%d\n", index);
}



//*****************************************************************************
static void _show_time_stamp(void)
{
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= (char*) asctime(stm);
	gsc_label("Time Stamp");
	printf("%s", psz);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall flow of the application.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	We tested a device.
*		EXIT_FAILURE	One or more problems were encounterred.
*
*****************************************************************************/

int main(int argc, char** argv)
{
	int		errs	= 0;
	int		fd		= 0;
	int		qty;
	int		ret;

	for (;;)
	{
		gsc_label_init(28);
		errs	= _parse_args(argc, argv);

		if (errs)
			break;

		// Introduction

		os_id_host();
		_show_time_stamp();
		_show_access_index(_def_index);
		errs	+= ao4mf_init_util(1);
		errs	+= os_id_driver(ao4mf_open, ao4mf_read, ao4mf_close);
		errs	+= ao4mf_count_boards(1, &qty);

		if ((errs) || (qty <= 0))
			break;

		// Open access to device

		errs	= ao4mf_open_util(_def_index, 0, -1, 1, &fd);

		if (errs == 0)
		{
			// Device identification
			// Perform testing

			errs	= _perform_tests(fd);

			// Close device access

			errs	+= ao4mf_close_util(fd, -1, 1);
		}

		// End of processing.

		break;
	}

	ret	= ((qty <= 0) || (errs)) ? EXIT_FAILURE : EXIT_SUCCESS;
	return(ret);
}



