// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/samples/signals/signals.c $
// $Rev: 43287 $
// $Date: 2018-07-09 17:43:15 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Sample Application: source file

#include "main.h"



/******************************************************************************
*
*	Function:	signals
*
*		Output the clocking signals for a period.
*
*	Arguments:
*
*		fd		The handle to the board to access.
*
*		seconds	Output the signal for this many seconds.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int signals(int fd, int seconds)
{
	static const u32 data[9]	=
	{
		0x0000, 0x2000, 0x4000, 0x6000,
		0x8000, 0xA000, 0xC000, 0xE000, 0x1FFFF
	};

	time_t	end;
	int		errs	= 0;
	time_t	now;
	int		ret;
	s32		val;

	for (;;)
	{
		gsc_label("Output Signals");
		val	= AO4MF_QUERY_SYNC_CLK_OUT;
		ret	= ao4mf_ioctl(fd, AO4MF_IOCTL_QUERY, &val);

		if (ret)
		{
			errs++;
			printf("FAIL <---  (%d. ERROR: ao4mf_ioctl())\n", __LINE__);
			break;
		}

		if (val == 0)
		{
			printf("SKIPPED  (This channel doesn't support the SYNC/CLK OUT signals.)\n");
			break;
		}

		printf("\n");
		gsc_label_level_inc();
		errs	+= ao4mf_initialize		(fd, -1, 1);
		errs	+= ao4mf_data_format	(fd, -1, 1, AO4MF_DATA_FORMAT_OFF_BIN, NULL);
		errs	+= ao4mf_ao_clk_enable	(fd, -1, 1, AO4MF_AO_CLK_ENABLE_NO, NULL);

		gsc_label("Writing Data");
		ret	= ao4mf_write(fd, data, sizeof(data));

		if (ret < 0)
		{
			// The error has already posted to the screen.
		}
		else if (ret == sizeof(data))
		{
			printf("PASS\n");
		}
		else
		{
			errs++;
			printf("FAIL <---  (%d. Didn't write all data.)\n", __LINE__);
		}

		errs	+= ao4mf_ao_buf_mode	(fd, -1, 1, AO4MF_AO_BUF_MODE_CIRC, NULL);

		//***********************************************************
		// The output clock signal.
		gsc_label("Clock Output");
		printf("\n");
		gsc_label_level_inc();
		errs	+= ao4mf_rate_gen_sel	(fd, -1, 1, AO4MF_RATE_GEN_SEL_CH_0, NULL);
		errs	+= ao4mf_int_samp_rate	(fd, -1, 1, AO4MF_INT_SAMP_RATE_224000_SPS, NULL);
		errs	+= ao4mf_ao_clk_src		(fd, -1, 1, AO4MF_AO_CLK_SRC_INT, NULL);
		errs	+= ao4mf_ao_clk_enable	(fd, -1, 1, AO4MF_AO_CLK_ENABLE_YES, NULL);
		gsc_label_level_dec();

		//***********************************************************
		// The output sync signal.
		gsc_label("Sync Output");
		printf("\n");
		gsc_label_level_inc();
		errs	+= ao4mf_burst_mode		(fd, -1, 1, AO4MF_BURST_MODE_ENABLE, NULL);

		gsc_label("Working");
		printf("%d seconds ... ", seconds);
		fflush(stdout);
		end	= time(NULL) + seconds;

		for (; errs == 0;)
		{
			errs	+= ao4mf_burst_trigger(fd, -1, 0);
			now		= time(NULL);

			if (now > end)
				break;

			gsc_time_sleep_ms(1);
		}

		printf("Done\n");
		gsc_label_level_dec();

		errs	+= ao4mf_initialize(fd, -1, 1);
		gsc_label_level_dec();
		break;
	}

	return(errs);
}



