// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/utils/util_ao_buf_size.c $
// $Rev: 43274 $
// $Date: 2018-07-09 17:36:40 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao4mf_ao_buf_size
*
*	Purpose:
*
*		Provide a visual wrapper for the AO4MF_IOCTL_AO_BUF_SIZE service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the value to apply.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao4mf_ao_buf_size(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs;
	const char*	ptr;
	int			ret;

	if (verbose)
		gsc_label_index("AO Buffer Size", index);

	ret		= ao4mf_ioctl(fd, AO4MF_IOCTL_AO_BUF_SIZE, &set);
	errs	= ret ? 1 : 0;

	switch (set)
	{
		default:

			errs++;
			ptr	= buf;
			sprintf(buf, "Unrecognized option: 0x%lX", (long) set);
			break;

		case AO4MF_AO_BUF_SIZE_4:

			ptr	= "4 Samples Deep";
			break;

		case AO4MF_AO_BUF_SIZE_8:

			ptr	= "8 Samples Deep";
			break;

		case AO4MF_AO_BUF_SIZE_16:

			ptr	= "16 Samples Deep";
			break;

		case AO4MF_AO_BUF_SIZE_32:

			ptr	= "32 Samples Deep";
			break;

		case AO4MF_AO_BUF_SIZE_64:

			ptr	= "64 Samples Deep";
			break;

		case AO4MF_AO_BUF_SIZE_128:

			ptr	= "128 Samples Deep";
			break;

		case AO4MF_AO_BUF_SIZE_256:

			ptr	= "256 Samples Deep";
			break;

		case AO4MF_AO_BUF_SIZE_512:

			ptr	= "512 Samples Deep";
			break;

		case AO4MF_AO_BUF_SIZE_1K:

			ptr	= "1K Samples Deep";
			break;

		case AO4MF_AO_BUF_SIZE_2K:

			ptr	= "2K Samples Deep";
			break;

		case AO4MF_AO_BUF_SIZE_4K:

			ptr	= "4K Samples Deep";
			break;

		case AO4MF_AO_BUF_SIZE_8K:

			ptr	= "8K Samples Deep";
			break;

		case AO4MF_AO_BUF_SIZE_16K:

			ptr	= "16K Samples Deep";
			break;

		case AO4MF_AO_BUF_SIZE_32K:

			ptr	= "32K Samples Deep";
			break;
	}

	if (verbose)
		printf("%s  (%s)\n", errs ? "FAIL <---" : "PASS", ptr);

	if (get)
		get[0]	= set;

	return(errs);
}


