// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/16AO4MF/16AO4MF_Linux_2.x.x.x_GSC_DN/utils/util_query.c $
// $Rev: 43277 $
// $Date: 2018-07-09 17:40:24 -0500 (Mon, 09 Jul 2018) $

// 16AO4MF: Utilities: source file

#include "main.h"



/******************************************************************************
*
*	Function:	ao4mf_query
*
*	Purpose:
*
*		Provide a visual wrapper for the AO4MF_IOCTL_QUERY service.
*
*	Arguments:
*
*		fd		Use this handle to access the device.
*
*		index	The index of the device to access. Ignore if < 0.
*				This is for display purposes only.
*
*		verbose	Work verbosely?
*
*		set		This is the query option to access.
*
*		get		The results are reported here. This may be NULL.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

int ao4mf_query(int fd, int index, int verbose, s32 set, s32* get)
{
	char		buf[128];
	int			errs	= 0;
	const char*	ptr;
	s32			query	= set;
	int			ret;

	if (verbose)
	{
		switch (query)
		{
			default:

				errs++;
				ptr	= buf;
				printf(buf, "Query Error (#%ld)", (long) query);
				break;

			case AO4MF_QUERY_AUT_CAL_MS:	ptr	= "Auto-Cal Period";	break;
			case AO4MF_QUERY_COUNT:			ptr	= "Query Options";		break;
			case AO4MF_QUERY_DEVICE_TYPE:	ptr	= "Device Type";		break;
			case AO4MF_QUERY_FREF_ADJ:		ptr	= "Fref Adjustable";	break;
			case AO4MF_QUERY_FREF_MASTER:	ptr	= "Master Clock";		break;
			case AO4MF_QUERY_FSAMP_MAX:		ptr	= "Fsamp Max";			break;
			case AO4MF_QUERY_FSAMP_MIN:		ptr	= "Fsamp Min";			break;
			case AO4MF_QUERY_INDEX_CHANNEL:	ptr = "Chan Index";			break;
			case AO4MF_QUERY_INDEX_DEVICE:	ptr = "Device Index";		break;
			case AO4MF_QUERY_INIT_MS:		ptr	= "Initialize Period";	break;
			case AO4MF_QUERY_NCLK_MAX:		ptr = "Nclk Mask";			break;
			case AO4MF_QUERY_NCLK_MIN:		ptr	= "Nclk Min";			break;
			case AO4MF_QUERY_NRATE_MAX:		ptr = "Nrate Max";			break;
			case AO4MF_QUERY_NRATE_MIN:		ptr = "Nrate Min";			break;
			case AO4MF_QUERY_REG_DIOPCR:	ptr	= "Reg: DIOPCR";		break;
			case AO4MF_QUERY_SYNC_CLK_OUT:	ptr	= "Sync Clk Out";		break;
		}

		gsc_label_index(ptr, index);
	}

	ret		= ao4mf_ioctl(fd, AO4MF_IOCTL_QUERY, &set);
	errs	= ret ? 1 : 0;

	if (verbose)
	{
		switch (query)
		{
			default:

				errs++;
				printf(buf, "%ld", (long) set);
				break;

			case AO4MF_QUERY_CHANNEL_QTY:
			case AO4MF_QUERY_COUNT:
			case AO4MF_QUERY_INDEX_CHANNEL:
			case AO4MF_QUERY_INDEX_DEVICE:
			case AO4MF_QUERY_NCLK_MAX:
			case AO4MF_QUERY_NCLK_MIN:
			case AO4MF_QUERY_NRATE_MAX:
			case AO4MF_QUERY_NRATE_MIN:

				sprintf(buf, "%ld", (long) set);
				break;

			case AO4MF_QUERY_FREF_ADJ:
			case AO4MF_QUERY_FREF_MASTER:

				sprintf(buf, "%ld Hz", (long) set);
				break;

			case AO4MF_QUERY_AUT_CAL_MS:
			case AO4MF_QUERY_INIT_MS:

				sprintf(buf, "%ld ms", (long) set);
				break;

			case AO4MF_QUERY_FSAMP_MAX:
			case AO4MF_QUERY_FSAMP_MIN:

				sprintf(buf, "%ld S/S", (long) set);
				break;

			case AO4MF_QUERY_REG_DIOPCR:
			case AO4MF_QUERY_SYNC_CLK_OUT:

				sprintf(buf, "%s", set ? "Supported" : "Not Supported");
				break;

			case AO4MF_QUERY_DEVICE_TYPE:

				if (set == GSC_DEV_TYPE_16AO4MF)
				{
					strcpy(buf, "16AO4MF");
				}
				else
				{
					errs++;
					sprintf(buf, "INVALID: %ld", (long) set);
				}

				break;
		}

		if (errs)
			printf("FAIL <---  (%s)\n", buf);
		else
			printf("%s\n", buf);
	}

	if (get)
		get[0]	= set;

	return(errs);
}


